@@ -1,26 +1,25 @@
 BUILD_BIT='64bit'
 BUILD_CPU='x86_64'
-BUILD_DATE='2022-03-06'
+BUILD_DATE='2022-06-21'
 BUILD_MACHINE='aarch64'
 BUILD_OS='linux'
-BUILD_TIME='23:13'
+BUILD_TIME='14:48'
 BUILT_BY='antoine'
 BUILT_ON='localhost.localdomain'
-COMPILER_VERSION='aarch64-unknown-linux-gnu-gcc (GCC) 11.2.0'
-CYTHON_VERSION='0.29.28'
+COMPILER_VERSION='aarch64-unknown-linux-gnu-gcc (GCC) 12.1.0'
+CYTHON_VERSION='0.29.30'
 LINKER_VERSION='GNU ld (GNU Binutils) 2.38'
-PYTHON_VERSION='3.10.2'
-RELEASE_BUILD=True
+PYTHON_VERSION='3.10.5'
 lib_gobject_introspection='1.68.0'
-lib_gtk='3.24.33'
-lib_gtk_x11='3.24.33'
+lib_gtk='3.24.34'
+lib_gtk_x11='3.24.34'
 lib_nvenc='10'
 lib_nvfbc='6.0'
-lib_py3cairo='1.20.1'
-lib_pygobject='3.42.0'
+lib_py3cairo='1.21.0'
+lib_pygobject='3.42.1'
 lib_python3='3.10'
 lib_vpx='1.11.0'
-lib_x11='1.7.3.1'
+lib_x11='1.8.1'
 lib_x264='0.164.3081M'
 lib_x265='3.5'
 lib_xcomposite='0.4.5'
