@@ -4,19 +4,30 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os.path
 import sys
+from time import monotonic
+
 from gi.repository import GLib
 from gi.repository import GObject
 
 from xpra.util import (
-    nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
-    disconnect_is_an_error, ellipsizer, DONE, first_time,
+    u, net_utf8, nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
+    disconnect_is_an_error, ellipsizer, first_time, csv,
+    repr_ellipsized,
+    SERVER_EXIT, DONE,
     )
-from xpra.os_util import bytestostr, strtobytes, get_hex_uuid, POSIX, OSX, hexstr
+from xpra.os_util import (
+    bytestostr,
+    get_hex_uuid, hexstr,
+    POSIX, OSX,
+    )
+from xpra.simple_stats import std_unit
 from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT
 from xpra.exit_codes import (
     EXIT_OK, EXIT_CONNECTION_LOST, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
     EXIT_FAILURE, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG,
+    EXIT_IO_ERROR, EXIT_NO_DATA,
     )
 from xpra.log import Logger
 
@@ -55,15 +66,8 @@ class GObjectXpraClient(GObject.GObject, XpraClientBase):
 
 
     def install_signal_handlers(self):
-        #only register the glib signal handler
-        #once the main loop is running,
-        #before that we just trigger a KeyboardInterrupt
-        def do_install_signal_handlers():
-            sstr = "%s Client" % self.client_type()
-            from xpra.gtk_common.gobject_compat import register_os_signals, register_SIGUSR_signals
-            register_os_signals(self.handle_app_signal, sstr)
-            register_SIGUSR_signals(sstr)
-        GLib.idle_add(do_install_signal_handlers)
+        from xpra.gtk_common.gobject_compat import install_signal_handlers
+        install_signal_handlers("%s Client" % self.client_type(), self.handle_app_signal)
 
 
     def setup_connection(self, conn):
@@ -96,22 +100,27 @@ class GObjectXpraClient(GObject.GObject, XpraClientBase):
 
     def run(self):
         XpraClientBase.run(self)
+        self.run_loop()
+        return self.exit_code
+
+    def run_loop(self):
         self.glib_mainloop = GLib.MainLoop()
         self.glib_mainloop.run()
-        return self.exit_code
 
     def make_hello(self):
         capabilities = XpraClientBase.make_hello(self)
         capabilities["keyboard"] = False
         return capabilities
 
-    def quit(self, exit_code):
+    def quit(self, exit_code=0):
         log("quit(%s) current exit_code=%s", exit_code, self.exit_code)
         if self.exit_code is None:
             self.exit_code = exit_code
         self.cleanup()
-        GLib.timeout_add(50, self.glib_mainloop.quit)
+        GLib.timeout_add(50, self.exit_loop)
 
+    def exit_loop(self):
+        self.glib_mainloop.quit()
 
 
 class CommandConnectClient(GObjectXpraClient):
@@ -128,6 +137,7 @@ class CommandConnectClient(GObjectXpraClient):
         #so don't try probing for printers, etc
         self.file_transfer = False
         self.printing = False
+        self.command_timeout = None
         #don't bother with many of these things for one-off commands:
         for x in ("ui_client", "wants_aliases", "wants_encodings",
                   "wants_versions", "wants_features", "wants_sound", "windows",
@@ -138,18 +148,30 @@ class CommandConnectClient(GObjectXpraClient):
     def setup_connection(self, conn):
         protocol = super().setup_connection(conn)
         if conn.timeout>0:
-            GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
+            self.command_timeout = GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
         return protocol
 
     def timeout(self, *_args):
         log.warn("timeout!")    # pragma: no cover
 
+    def cancel_command_timeout(self):
+        ct = self.command_timeout
+        if ct:
+            self.command_timeout = None
+            GLib.source_remove(ct)
 
-    def _process_connection_lost(self, _packet):
+    def _process_connection_lost(self, packet):
+        log("_process_connection_lost%s", packet)
         #override so we don't log a warning
         #"command clients" are meant to exit quickly by losing the connection
         p = self._protocol
         if p and p.input_packetcount==0:
+            #never got any data back so we have never connected,
+            #try to give feedback to the user as to why that is:
+            details = ""
+            if len(packet)>1:
+                details = ": %s" % csv(packet[1:])
+            log.warn("Connection failed%s", details)
             self.quit(EXIT_CONNECTION_LOST)
         else:
             self.quit(EXIT_OK)
@@ -160,6 +182,7 @@ class CommandConnectClient(GObjectXpraClient):
         #* we don't care about sending anything back after hello
         log("server_capabilities: %s", ellipsizer(caps))
         log("protocol state: %s", self._protocol.save_state())
+        self.cancel_command_timeout()
         self.do_command(caps)
         return True
 
@@ -179,6 +202,9 @@ class SendCommandConnectClient(CommandConnectClient):
         assert self.parse_network_capabilities(caps), "network capabilities failure"
         return super().server_connection_established(caps)
 
+    def do_command(self, caps : typedict):
+        raise NotImplementedError()
+
 
 class HelloRequestClient(SendCommandConnectClient):
     """
@@ -231,13 +257,18 @@ class ScreenshotXpraClient(CommandConnectClient):
 
     def _process_screenshot(self, packet):
         (w, h, encoding, _, img_data) = packet[1:6]
-        assert encoding==b"png", "expected png screenshot data but got %s" % bytestostr(encoding)
+        assert bytestostr(encoding)=="png", "expected png screenshot data but got %s" % bytestostr(encoding)
         if not img_data:
             self.warn_and_quit(EXIT_OK,
                                "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
             return
-        with open(self.screenshot_filename, 'wb') as f:
-            f.write(img_data)
+        if self.screenshot_filename=="-":
+            output = os.fdopen(sys.stdout.fileno(), "wb", closefd=False)
+        else:
+            output = open(self.screenshot_filename, "wb")
+        with output:
+            output.write(img_data)
+            output.flush()
         self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
 
     def init_packet_handlers(self):
@@ -261,7 +292,13 @@ class InfoXpraClient(CommandConnectClient):
         self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the info")
 
     def do_command(self, caps : typedict):
-        if caps:
+        def print_fn(s):
+            sys.stdout.write("%s\n" % (s,))
+        if not caps:
+            self.quit(EXIT_NO_DATA)
+            return
+        exit_code = EXIT_OK
+        try:
             if FLATTEN_INFO<2:
                 #compatibility mode:
                 c = flatten_dict(caps)
@@ -276,19 +313,18 @@ class InfoXpraClient(CommandConnectClient):
                         if isinstance(w, bytes):
                             if k.endswith(".data"):
                                 return hexstr(w)
-                            try:
-                                return w.decode("utf-8")
-                            except:
-                                return bytestostr(w)
+                            return u(w)
                         elif isinstance(w, (tuple,list)):
                             return type(w)([fixvalue(x) for x in w])
                         return w
                     v = fixvalue(v)
                     k = fixvalue(k)
-                    log.info("%s=%s", k, nonl(v))
+                    print_fn("%s=%s" % (k, nonl(v)))
             else:
-                print_nested_dict(caps)
-        self.quit(EXIT_OK)
+                print_nested_dict(caps, print_fn=print_fn)
+        except OSError:
+            exit_code = EXIT_IO_ERROR
+        self.quit(exit_code)
 
 class IDXpraClient(InfoXpraClient):
 
@@ -297,6 +333,16 @@ class IDXpraClient(InfoXpraClient):
         self.hello_extra["request"] = "id"
 
 
+class RequestXpraClient(CommandConnectClient):
+
+    def __init__(self, request, opts):
+        super().__init__(opts)
+        self.hello_extra["request"] = request
+
+    def do_command(self, caps : typedict):
+        self.quit(EXIT_OK)
+
+
 class ConnectTestXpraClient(CommandConnectClient):
     """ This client does one thing only:
         it queries the server with an 'info' request
@@ -370,6 +416,186 @@ class MonitorXpraClient(SendCommandConnectClient):
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
 
+class InfoTimerClient(MonitorXpraClient):
+    """
+        This client keeps monitoring the server
+        and requesting info data
+    """
+    REFRESH_RATE = envint("XPRA_REFRESH_RATE", 1)
+
+    def __init__(self, *args):
+        super().__init__(*args)
+        self.info_request_pending = False
+        self.server_last_info = typedict()
+        self.server_last_info_time = 0
+        self.info_timer = 0
+
+    def run(self):
+        from xpra.gtk_common.gobject_compat import register_os_signals
+        register_os_signals(self.signal_handler, None)
+        v = super().run()
+        self.log("run()=%s" % v)
+        self.cleanup()
+        return v
+
+    def signal_handler(self, signum, *args):
+        self.log("exit_code=%s" % self.exit_code)
+        self.log("signal_handler(%s, %s)" % (signum, args,))
+        self.quit(128+signum)
+        self.log("exit_code=%s" % self.exit_code)
+
+    def log(self, message):
+        #this method is overriden in top client to use a log file
+        log(message)
+
+    def err(self, e):
+        log.error(str(e))
+
+    def cleanup(self):
+        self.cancel_info_timer()
+        MonitorXpraClient.cleanup(self)
+
+    def do_command(self, caps : typedict):
+        self.send_info_request()
+        self.timeout_add(self.REFRESH_RATE*1000, self.send_info_request)
+
+    def send_info_request(self, *categories):
+        self.log("send_info_request%s" % (categories,))
+        if not self.info_request_pending:
+            self.info_request_pending = True
+            window_ids = ()    #no longer used or supported by servers
+            self.send("info-request", [self.uuid], window_ids, categories)
+        if not self.info_timer:
+            self.info_timer = self.timeout_add((self.REFRESH_RATE+2)*1000, self.info_timeout)
+        return True
+
+    def init_packet_handlers(self):
+        MonitorXpraClient.init_packet_handlers(self)
+        self.add_packet_handler("info-response", self._process_info_response, False)
+
+    def _process_server_event(self, packet):
+        self.log("server event: %s" % (packet,))
+        self.last_server_event = packet[1:]
+        self.update_screen()
+
+    def _process_info_response(self, packet):
+        self.log("info response: %s" % repr_ellipsized(packet))
+        self.cancel_info_timer()
+        self.info_request_pending = False
+        self.server_last_info = typedict(packet[1])
+        self.server_last_info_time = monotonic()
+        #log.info("server_last_info=%s", self.server_last_info)
+        self.update_screen()
+
+    def cancel_info_timer(self):
+        it = self.info_timer
+        if it:
+            self.info_timer = None
+            self.source_remove(it)
+
+    def info_timeout(self):
+        self.log("info timeout")
+        self.update_screen()
+        return True
+
+    def update_screen(self):
+        raise NotImplementedError()
+
+
+class ShellXpraClient(SendCommandConnectClient):
+    """
+        Provides an interactive shell with the socket it connects to
+    """
+
+    def __init__(self, opts):
+        super().__init__(opts)
+        self.stdin_io_watch = None
+        self.stdin_buffer = ""
+        self.hello_extra["shell"] = "True"
+
+    def timeout(self, *args):
+        """
+        The shell client never times out,
+        but the superclass calls this method automatically,
+        just ignore it.
+        """
+
+    def cleanup(self):
+        siw = self.stdin_io_watch
+        if siw:
+            self.stdin_io_watch = None
+            self.source_remove(siw)
+        super().cleanup()
+
+    def do_command(self, caps : typedict):
+        if not caps.boolget("shell"):
+            msg = "this server does not support the 'shell' subcommand"
+            log.error(msg)
+            self.disconnect_and_quit(EXIT_UNSUPPORTED, msg)
+            return
+        #start reading from stdin:
+        self.install_signal_handlers()
+        stdin = sys.stdin
+        fileno = stdin.fileno()
+        import fcntl
+        fl = fcntl.fcntl(fileno, fcntl.F_GETFL)
+        fcntl.fcntl(fileno, fcntl.F_SETFL, fl | os.O_NONBLOCK)
+        self.stdin_io_watch = GLib.io_add_watch(sys.stdin,
+                                                GLib.PRIORITY_DEFAULT, GLib.IO_IN,
+                                                self.stdin_ready)
+        self.print_prompt()
+
+    def stdin_ready(self, *_args):
+        data = sys.stdin.read()
+        #log.warn("stdin=%r", data)
+        self.stdin_buffer += data
+        sent = 0
+        if self.stdin_buffer.endswith("\n"):
+            for line in self.stdin_buffer.splitlines():
+                if line:
+                    if line.rstrip("\n\r") in ("quit", "exit"):
+                        self.disconnect_and_quit(EXIT_OK, "user requested %s" % line)
+                        self.stdin_io_watch = None
+                        return False
+                    self.send("shell-exec", line.encode())
+                    sent += 1
+        self.stdin_buffer = ""
+        if not sent:
+            self.print_prompt()
+        return True
+
+    def init_packet_handlers(self):
+        super().init_packet_handlers()
+        self._packet_handlers["shell-reply"] = self._process_shell_reply
+        self._packet_handlers["ping"] = self._process_ping
+
+    def _process_ping(self, packet):
+        echotime = packet[1]
+        self.send("ping_echo", echotime, 0, 0, 0, -1)
+
+    def _process_shell_reply(self, packet):
+        fd = packet[1]
+        message = packet[2]
+        if fd==1:
+            stream = sys.stdout
+        elif fd==2:
+            stream = sys.stderr
+        else:
+            raise Exception("invalid file descriptor %i" % fd)
+        s = net_utf8(message)
+        if s.endswith("\n"):
+            s = s[:-1]
+        stream.write("%s" % s)
+        stream.flush()
+        if fd==2:
+            stream.write("\n")
+            self.print_prompt()
+
+    def print_prompt(self):
+        sys.stdout.write("> ")
+        sys.stdout.flush()
+
+
 class VersionXpraClient(HelloRequestClient):
     """ This client does one thing only:
         it queries the server for version information and prints it out
@@ -387,11 +613,12 @@ class VersionXpraClient(HelloRequestClient):
         return True
 
     def do_command(self, caps : typedict):
-        v = caps.strget(b"version")
+        v = caps.strget("version")
         if not v:
             self.warn_and_quit(EXIT_FAILURE, "server did not provide the version information")
         else:
             sys.stdout.write("%s\n" % (v,))
+            sys.stdout.flush()
             self.quit(EXIT_OK)
 
 
@@ -420,12 +647,7 @@ class ControlXpraClient(CommandConnectClient):
     def make_hello(self):
         capabilities = super().make_hello()
         log("make_hello() adding command request '%s' to %s", self.command, capabilities)
-        def b(s):
-            try:
-                return s.encode("utf8")
-            except:
-                return strtobytes(s)
-        capabilities["command_request"] = tuple(b(x) for x in self.command)
+        capabilities["command_request"] = tuple(self.command)
         capabilities["request"] = "command"
         return capabilities
 
@@ -442,7 +664,8 @@ class PrintClient(SendCommandConnectClient):
         #TODO: load as needed...
         def sizeerr(size):
             self.warn_and_quit(EXIT_FILE_TOO_BIG,
-                               "the file is too large: %iMB (the file size limit is %iMB)" % (size//1024//1024, self.file_size_limit))
+                               "the file is too large: %sB (the file size limit is %sB)" % (
+                                   std_unit(size), std_unit(self.file_size_limit)))
             return
         if self.filename=="-":
             #replace with filename proposed
@@ -452,16 +675,15 @@ class PrintClient(SendCommandConnectClient):
                 self.file_data = stdin_binary.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
-            import os.path
             size = os.path.getsize(self.filename)
-            if size>self.file_size_limit*1024*1024:
+            if size>self.file_size_limit:
                 sizeerr(size)
                 return
             from xpra.os_util import load_binary_file
             self.file_data = load_binary_file(self.filename)
             log("read %i bytes from %s", len(self.file_data), self.filename)
         size = len(self.file_data)
-        if size>self.file_size_limit*1024*1024:
+        if size>self.file_size_limit:
             sizeerr(size)
             return
         assert self.file_data, "no data found for '%s'" % self.filename
@@ -477,8 +699,9 @@ class PrintClient(SendCommandConnectClient):
         if not printing:
             self.warn_and_quit(EXIT_UNSUPPORTED, "server does not support printing")
             return
-        #TODO: compress file data? (this should run locally most of the time anyway)
-        from xpra.net.compression import Compressed
+        #we don't compress file data
+        #(this should run locally most of the time anyway)
+        from xpra.net.compression import Compressed  #pylint: disable=import-outside-toplevel
         blob = Compressed("print", self.file_data)
         self.send("print", self.filename, blob, *self.command)
         log("print: sending %s as %s for printing", self.filename, blob)
@@ -505,7 +728,7 @@ class ExitXpraClient(HelloRequestClient):
             }
 
     def do_command(self, caps : typedict):
-        self.idle_add(self.send, "exit-server")
+        self.idle_add(self.send, "exit-server", os.environ.get("XPRA_EXIT_MESSAGE", SERVER_EXIT))
 
 
 class StopXpraClient(HelloRequestClient):
@@ -590,7 +813,7 @@ class RequestStartClient(HelloRequestClient):
                 }.get(mode, "")
             try:
                 errwrite("\n%ssession now available on display %s\n" % (session_type, display))
-                if POSIX and not OSX and self.displayfd>0 and display and display.startswith(b":"):
+                if POSIX and not OSX and self.displayfd>0 and display and display.startswith(":"):
                     from xpra.platform.displayfd import write_displayfd
                     log("writing display %s to displayfd=%s", display, self.displayfd)
                     write_displayfd(self.displayfd, display[1:])
