@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -7,6 +7,7 @@
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
 from xpra.platform.features import CLIPBOARD_WANT_TARGETS, CLIPBOARD_GREEDY, CLIPBOARD_PREFERRED_TARGETS, CLIPBOARDS
 from xpra.platform.gui import get_clipboard_native_class
+from xpra.net import compression
 from xpra.scripts.config import FALSE_OPTIONS, TRUE_OPTIONS
 from xpra.util import flatten_dict, typedict
 from xpra.os_util import bytestostr, is_Wayland
@@ -22,18 +23,19 @@ class ClipboardClient(StubClientMixin):
     __signals__ = ["clipboard-toggled"]
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         self.client_clipboard_type = ""
         self.client_clipboard_direction = "both"
         self.client_supports_clipboard = False
         self.clipboard_enabled = False
         self.server_clipboard_direction = "both"
         self.server_clipboard = False
-        self.server_clipboard_loop_uuids = {}
         self.server_clipboard_direction = ""
-        self.server_clipboard_contents_slice_fix = False
-        self.server_clipboard_preferred_targets = False
-        self.server_clipboards = []
+        self.server_clipboard_preferred_targets = ()
+        self.server_clipboards = ()
+        self.server_clipboard_greedy = False
+        self.server_clipboard_want_targets = False
+        self.server_clipboard_selections = ()
         self.clipboard_helper = None
         self.local_clipboard_requests = 0
         self.remote_clipboard_requests = 0
@@ -94,7 +96,7 @@ class ClipboardClient(StubClientMixin):
                 "greedy"                    : CLIPBOARD_GREEDY,
                 "preferred-targets"         : CLIPBOARD_PREFERRED_TARGETS,
                 "set_enabled"               : True,     #v4 servers no longer use or show this flag
-                "contents-slice-fix"        : True,     #fixed in v2.4
+                "contents-slice-fix"        : True,     #fixed in v2.4, removed check in v4.3
                 },
              })
         return caps
@@ -108,7 +110,6 @@ class ClipboardClient(StubClientMixin):
             self.clipboard_enabled = False
             return True
         self.server_clipboard = c.boolget("clipboard")
-        self.server_clipboard_loop_uuids = c.dictget("clipboard.loop-uuids", {})
         self.server_clipboard_direction = c.strget("clipboard-direction", "both")
         if self.server_clipboard_direction!=self.client_clipboard_direction and self.server_clipboard_direction!="both":
             if self.client_clipboard_direction=="disabled":
@@ -133,11 +134,14 @@ class ClipboardClient(StubClientMixin):
         log("client clipboard: supported=%s, direction=%s",
                      self.client_supports_clipboard, self.client_clipboard_direction)
         self.clipboard_enabled = self.client_supports_clipboard and self.server_clipboard
+        self.server_clipboard_greedy = c.boolget("clipboard.greedy")
+        self.server_clipboard_want_targets = c.boolget("clipboard.want_targets")
+        self.server_clipboard_selections = c.strtupleget("clipboard.selections", CLIPBOARDS)
+        self.server_clipboard_preferred_targets = c.strtupleget("clipboard.preferred-targets", ())
+        log("server clipboard: greedy=%s, want_targets=%s, selections=%s",
+            self.server_clipboard_greedy, self.server_clipboard_want_targets, self.server_clipboard_selections)
         log("parse_clipboard_caps() clipboard enabled=%s", self.clipboard_enabled)
-        self.server_clipboard_contents_slice_fix = c.boolget("clipboard.contents-slice-fix")
         self.server_clipboard_preferred_targets = c.strtupleget("clipboard.preferred-targets", ())
-        if not self.server_clipboard_contents_slice_fix:
-            log.info("server clipboard does not include contents slice fix")
         return True
 
     def process_ui_capabilities(self, caps : typedict):
@@ -148,8 +152,6 @@ class ClipboardClient(StubClientMixin):
                 log.warn("Warning: no clipboard support")
                 if is_Wayland():
                     log.warn(" (wayland display)")
-            else:
-                ch.set_clipboard_contents_slice_fix(self.server_clipboard_contents_slice_fix)
             self.clipboard_helper = ch
             self.clipboard_enabled = ch is not None
             log("clipboard helper=%s", ch)
@@ -257,13 +259,6 @@ class ClipboardClient(StubClientMixin):
         log("send_clipboard_selections(%s)", selections)
         self.send_now("clipboard-enable-selections", selections)
 
-    def send_clipboard_loop_uuids(self):
-        uuids = self.clipboard_helper.get_loop_uuids()
-        log("send_clipboard_loop_uuid() uuids=%s", uuids)
-        if self.server_clipboard_loop_uuids:
-            self.send_now("clipboard-loop-uuids", uuids)
-
-
     def setup_clipboard_helper(self, helperClass):
         log("setup_clipboard_helper(%s)", helperClass)
         #first add the platform specific one, (may be None):
@@ -274,7 +269,6 @@ class ClipboardClient(StubClientMixin):
                  "clipboards.remote"    : self.server_clipboards,
                  "can-send"             : self.client_clipboard_direction in ("to-server", "both"),
                  "can-receive"          : self.client_clipboard_direction in ("to-client", "both"),
-                 "remote-loop-uuids"    : self.server_clipboard_loop_uuids,
                  #the local clipboard we want to sync to (with the translated clipboard only):
                  "clipboard.local"      : self.local_clipboard,
                  #the remote clipboard we want to we sync to (with the translated clipboard only):
@@ -286,24 +280,14 @@ class ClipboardClient(StubClientMixin):
             if not self.clipboard_enabled:
                 log("clipboard is disabled, not sending clipboard packet")
                 return
-            #handle clipboard compression if needed:
-            from xpra.net.compression import Compressible
+            #replaces 'Compressible' items in a packet
+            #with a subclass that calls self.compressed_wrapper
+            #and which can therefore enable the brotli compressor:
             packet = list(parts)
-            for v in packet:
-                if isinstance(v, Compressible):
-                    register_clipboard_compress_cb(v)
+            for i, v in enumerate(packet):
+                if isinstance(v, compression.Compressible):
+                    packet[i] = self.compressible_item(v)
             self.send_now(*packet)
-        def register_clipboard_compress_cb(compressible):
-            #register the compressor which will fire in protocol.encode:
-            def compress_clipboard():
-                log("compress_clipboard() compressing %s, server compressors=%s",
-                                  compressible, self.server_compressors)
-                from xpra.net import compression
-                if "brotli" in self.server_compressors and compression.use("brotli"):
-                    return compression.compressed_wrapper(compressible.datatype, compressible.data,
-                                                        level=9, brotli=True, can_inline=False)
-                return self.compressed_wrapper(compressible.datatype, compressible.data)
-            compressible.compress = compress_clipboard
         def clipboard_progress(local_requests, remote_requests):
             log("clipboard_progress(%s, %s)", local_requests, remote_requests)
             if local_requests is not None:
@@ -314,7 +298,24 @@ class ClipboardClient(StubClientMixin):
             self.clipboard_notify(n)
         hc = helperClass(clipboard_send, clipboard_progress, **kwargs)
         hc.set_preferred_targets(self.server_clipboard_preferred_targets)
+        hc.set_greedy_client(self.server_clipboard_greedy)
+        hc.set_want_targets_client(self.server_clipboard_want_targets)
+        hc.enable_selections(self.server_clipboard_selections)
         return hc
 
+    def compressible_item(self, compressible):
+        """
+            converts a 'Compressible' item into something that will
+            call `self.compressed_wrapper` when compression is requested
+            by the network encode thread.
+        """
+        client = self
+        class ProtocolCompressible(compression.Compressible):
+            __slots__ = ()
+            def compress(self):
+                return client.compressed_wrapper(self.datatype, self.data,
+                                                 level=9, can_inline=False, brotli=True)
+        return ProtocolCompressible(compressible.datatype, compressible.data)
+
     def clipboard_notify(self, n):
         log("clipboard_notify(%i)", n)
