@@ -55,7 +55,7 @@ def get_core_encodings():
     for encoding in video_decodings:
         if encoding not in core_encodings:
             core_encodings.append(encoding)
-    #remove duplicates and use prefered encoding order:
+    #remove duplicates and use preferred encoding order:
     return [x for x in PREFERRED_ENCODING_ORDER if x in set(core_encodings)]
 
 
@@ -65,7 +65,7 @@ class Encodings(StubClientMixin):
     """
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         self.allowed_encodings = []
         self.encoding = None
         self.quality = -1
@@ -98,11 +98,14 @@ class Encodings(StubClientMixin):
         self.min_speed = opts.min_speed
         load_codec("dec_pillow")
         ae = self.allowed_encodings
+        if "png" in ae:
+            #try to load the fast png decoder:
+            load_codec("dec_spng")
         if "jpeg" in ae:
-            #try to load the fast jpeg encoder:
+            #try to load the fast jpeg decoder:
             load_codec("dec_jpeg")
         if "webp" in ae:
-            #try to load the fast webp encoder:
+            #try to load the fast webp decoder:
             load_codec("dec_webp")
         vh = getVideoHelper()
         vh.set_modules(video_decoders=opts.video_decoders, csc_modules=opts.csc_modules or NO_GFX_CSC_OPTIONS)
@@ -231,6 +234,10 @@ class Encodings(StubClientMixin):
                 full_csc_modes["webp"] = ("BGRX", "BGRA", "RGBX", "RGBA")
             else:
                 full_csc_modes["webp"] = ("BGRX", "BGRA", )
+        if has_codec("dec_jpeg") or has_codec("dec_pillow"):
+            full_csc_modes["jpeg"] = ("BGRX", "BGRA", "YUV420P")
+        if has_codec("dec_jpeg"):
+            full_csc_modes["jpega"] = ("BGRA", "RGBA", )
         log("supported full csc_modes=%s", full_csc_modes)
         caps["full_csc_modes"] = full_csc_modes
 
@@ -244,7 +251,7 @@ class Encodings(StubClientMixin):
             # so we don't bother specifying anything for those two.
             h264_caps = {}
             for csc_name, default_profile in (
-                        ("YUV420P", "high10"),
+                        ("YUV420P", "high"),
                         ("YUV422P", ""),
                         ("YUV444P", "")):
                 profile = os.environ.get("XPRA_H264_%s_PROFILE" % (csc_name), default_profile)
