@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,8 +11,9 @@ import os
 import errno
 import signal
 import datetime
+import math
 from collections import deque
-from time import sleep, time
+from time import sleep, time, monotonic
 from queue import Queue
 from gi.repository import GLib
 
@@ -23,14 +24,14 @@ from xpra.platform.gui import (
 from xpra.platform.features import SYSTEM_TRAY_SUPPORTED
 from xpra.platform.paths import get_icon_filename
 from xpra.scripts.config import FALSE_OPTIONS
-from xpra.make_thread import make_thread
+from xpra.make_thread import start_thread
 from xpra.os_util import (
-    bytestostr, monotonic_time, memoryview_to_bytes,
+    bytestostr, memoryview_to_bytes,
     OSX, POSIX, is_Ubuntu,
     )
 from xpra.util import (
-    iround, envint, envbool, typedict,
-    make_instance, updict, repr_ellipsized, csv,
+    envint, envbool, typedict,
+    make_instance, updict, repr_ellipsized, u, noerr,
     )
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
 from xpra.log import Logger
@@ -75,19 +76,21 @@ SAVE_CURSORS = envbool("XPRA_SAVE_CURSORS", False)
 SIGNAL_WATCHER = envbool("XPRA_SIGNAL_WATCHER", True)
 
 FAKE_SUSPEND_RESUME = envint("XPRA_FAKE_SUSPEND_RESUME", 0)
+MOUSE_SCROLL_SQRT_SCALE = envbool("XPRA_MOUSE_SCROLL_SQRT_SCALE", OSX)
+MOUSE_SCROLL_MULTIPLIER = envint("XPRA_MOUSE_SCROLL_MULTIPLIER", 100)
 
 
 DRAW_TYPES = {bytes : "bytes", str : "bytes", tuple : "arrays", list : "arrays"}
 
 
-"""
-Utility superclass for clients that handle windows:
-create, resize, paint, grabs, cursors, etc
-"""
 class WindowClient(StubClientMixin):
+    """
+    Utility superclass for clients that handle windows:
+    create, resize, paint, grabs, cursors, etc
+    """
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         self._window_to_id = {}
         self._id_to_window = {}
 
@@ -164,9 +167,9 @@ class WindowClient(StubClientMixin):
             if v:
                 try:
                     pv = tuple(int(x.strip()) for x in v.split("x", 1))
-                    assert len(pv)==2
-                    return pv
-                except:
+                    if len(pv)==2:
+                        return pv
+                except ValueError:
                     #the main script does some checking, but we could be called from a config file launch
                     log.warn("Warning: invalid window %s specified: %s", attribute, v)
             return None
@@ -215,7 +218,12 @@ class WindowClient(StubClientMixin):
         mouselog("wheel_map(%s)=%s", mw, self.wheel_map)
 
         if 0<ICON_OVERLAY<=100:
-            icon_filename = get_icon_filename("xpra")
+            icon_filename = opts.tray_icon
+            if icon_filename and not os.path.isabs(icon_filename):
+                icon_filename = get_icon_filename(icon_filename)
+            if not icon_filename or not os.path.exists(icon_filename):
+                icon_filename = get_icon_filename("xpra")
+            traylog("window icon overlay: %s", icon_filename)
             if icon_filename:
                 try:
                     #make sure Pillow's PNG image loader doesn't spam the output with debug messages:
@@ -228,7 +236,6 @@ class WindowClient(StubClientMixin):
                     log.error(" %s", e)
         traylog("overlay_image=%s", self.overlay_image)
         self._draw_queue = Queue()
-        self._draw_thread = make_thread(self._draw_thread_loop, "draw")
 
 
     def parse_border(self):
@@ -238,7 +245,7 @@ class WindowClient(StubClientMixin):
 
     def run(self):
         #we decode pixel data in this thread
-        self._draw_thread.start()
+        self._draw_thread = start_thread(self._draw_thread_loop, "draw")
         if FAKE_SUSPEND_RESUME:
             self.timeout_add(FAKE_SUSPEND_RESUME*1000, self.suspend)
             self.timeout_add(FAKE_SUSPEND_RESUME*1000*2, self.resume)
@@ -256,6 +263,10 @@ class WindowClient(StubClientMixin):
         self.cancel_lost_focus_timer()
         if dq:
             dq.put(None)
+        dt = self._draw_thread
+        log("WindowClient.cleanup() draw thread=%s, alive=%s", dt, dt and dt.is_alive())
+        if dt and dt.is_alive():
+            dt.join(0.1)
         log("WindowClient.cleanup() done")
 
 
@@ -281,8 +292,8 @@ class WindowClient(StubClientMixin):
             "draw-counter"  : self._draw_counter,
             "read-only"     : self.readonly,
             "wheel" : {
-                "delta-x"   : self.wheel_deltax,
-                "delta-y"   : self.wheel_deltay,
+                "delta-x"   : int(self.wheel_deltax*1000),
+                "delta-y"   : int(self.wheel_deltay*1000),
             },
             "focused"       : self._focused or 0,
             "grabbed"       : self._window_with_grab or 0,
@@ -336,12 +347,7 @@ class WindowClient(StubClientMixin):
         self.default_cursor_data = c.tupleget("cursor.default", None)
         self.server_bell = c.boolget("bell")          #added in 0.5, default to True!
         self.bell_enabled = self.server_bell and self.client_supports_bell
-        if c.boolget("windows", True):
-            if self.windows_enabled:
-                server_auto_refresh_delay = c.intget("auto_refresh_delay", 0)/1000.0
-                if server_auto_refresh_delay==0 and self.auto_refresh_delay>0:
-                    log.warn("Warning: server does not support auto-refresh!")
-        else:
+        if not c.boolget("windows", True):
             log.warn("Warning: window forwarding is not enabled on this server")
         self.server_window_signals = c.strtupleget("window.signals")
         self.server_window_states = c.strtupleget("window.states", (
@@ -366,7 +372,7 @@ class WindowClient(StubClientMixin):
         else:
             rx, ry = -1, -1
         cx, cy = self.get_mouse_position()
-        start_time = monotonic_time()
+        start_time = monotonic()
         mouselog("process_pointer_position: %i,%i (%i,%i relative to wid %i) - current position is %i,%i",
                  x, y, rx, ry, wid, cx, cy)
         size = 10
@@ -381,15 +387,14 @@ class WindowClient(StubClientMixin):
                     value = None
                 show_pointer_overlay(value)
 
-    def send_wheel_delta(self, wid, button, distance, *args):
+    def send_wheel_delta(self, wid, button, distance, pointer=None, *args):
         modifiers = self.get_current_modifiers()
-        pointer = self.get_mouse_position()
         buttons = []
         mouselog("send_wheel_delta(%i, %i, %.4f, %s) precise wheel=%s, modifiers=%s, pointer=%s",
                  wid, button, distance, args, self.server_precise_wheel, modifiers, pointer)
         if self.server_precise_wheel:
             #send the exact value multiplied by 1000 (as an int)
-            idist = int(distance*1000)
+            idist = round(distance*1000)
             if abs(idist)>0:
                 packet =  ["wheel-motion", wid,
                            button, idist,
@@ -401,14 +406,25 @@ class WindowClient(StubClientMixin):
             #server cannot handle precise wheel,
             #so we have to use discrete events,
             #and send a click for each step:
-            steps = abs(int(distance))
+            scaled_distance = abs(distance*MOUSE_SCROLL_MULTIPLIER/100)
+            if MOUSE_SCROLL_SQRT_SCALE:
+                scaled_distance = math.sqrt(scaled_distance)
+            steps = round(scaled_distance)
             for _ in range(steps):
                 self.send_button(wid, button, True, pointer, modifiers, buttons)
                 self.send_button(wid, button, False, pointer, modifiers, buttons)
             #return remainder:
-            return float(distance) - int(distance)
+            scaled_remainder = steps
+            if MOUSE_SCROLL_SQRT_SCALE:
+                scaled_remainder = steps**2
+            scaled_remainder = scaled_remainder*(100/float(MOUSE_SCROLL_MULTIPLIER))
 
-    def wheel_event(self, wid, deltax=0, deltay=0, deviceid=0):
+            remain_distance = float(scaled_remainder)
+            signed_remain_distance = remain_distance * (-1 if distance < 0 else 1)
+            return float(distance) - signed_remain_distance
+
+
+    def wheel_event(self, wid, deltax=0, deltay=0, pointer=(), deviceid=0):
         #this is a different entry point for mouse wheel events,
         #which provides finer grained deltas (if supported by the server)
         #accumulate deltas:
@@ -416,10 +432,10 @@ class WindowClient(StubClientMixin):
         self.wheel_deltay += deltay
         button = self.wheel_map.get(6+int(self.wheel_deltax>0))            #RIGHT=7, LEFT=6
         if button>0:
-            self.wheel_deltax = self.send_wheel_delta(wid, button, self.wheel_deltax, deviceid)
+            self.wheel_deltax = self.send_wheel_delta(wid, button, self.wheel_deltax, pointer, deviceid)
         button = self.wheel_map.get(5-int(self.wheel_deltay>0))            #UP=4, DOWN=5
         if button>0:
-            self.wheel_deltay = self.send_wheel_delta(wid, button, self.wheel_deltay, deviceid)
+            self.wheel_deltay = self.send_wheel_delta(wid, button, self.wheel_deltay, pointer, deviceid)
         mouselog("wheel_event%s new deltas=%s,%s",
                  (wid, deltax, deltay, deviceid), self.wheel_deltax, self.wheel_deltay)
 
@@ -438,7 +454,7 @@ class WindowClient(StubClientMixin):
     def scale_pointer(self, pointer):
         #subclass may scale this:
         #return int(pointer[0]/self.xscale), int(pointer[1]/self.yscale)
-        return int(pointer[0]), int(pointer[1])
+        return round(pointer[0]), round(pointer[1])
 
     def send_input_devices(self, fmt, input_devices):
         assert self.server_input_devices
@@ -456,29 +472,17 @@ class WindowClient(StubClientMixin):
         else:
             if len(packet)<9:
                 raise Exception("invalid cursor packet: %s items" % len(packet))
-            encoding = packet[1]
-            if isinstance(encoding, bytes):
-                encoding = encoding.decode("latin1")
-            if not isinstance(encoding, str):
-                log.warn("Warning: received an invalid cursor packet:")
-                tmp_packet = list(packet)
-                try:
-                    tmp_packet[9] = ".."
-                except IndexError:
-                    pass
-                log.warn(" %s", repr_ellipsized(tmp_packet))
-                log.warn(" data types:")
-                log.warn(" %s", csv(type(x) for x in packet))
-                raise Exception("invalid cursor packet format: cursor type is a %s" % type(encoding))
             #trim packet-type:
             new_cursor = packet[1:]
-            pixels = new_cursor[8]
+            encoding = u(new_cursor[0])
+            new_cursor[0] = encoding
             if encoding=="png":
+                pixels = new_cursor[8]
                 if SAVE_CURSORS:
                     serial = new_cursor[7]
                     with open("raw-cursor-%#x.png" % serial, 'wb') as f:
                         f.write(pixels)
-                from xpra.codecs.pillow.decoder import open_only
+                from xpra.codecs.pillow.decoder import open_only  #pylint: disable=import-outside-toplevel
                 img = open_only(pixels, ("png",))
                 new_cursor[8] = img.tobytes("raw", "BGRA")
                 cursorlog("used PIL to convert png cursor to raw")
@@ -948,7 +952,7 @@ class WindowClient(StubClientMixin):
             ww, wh = new_size_fn(ww, wh)
         try:
             bw, bh = window._backing.size
-        except:
+        except (AttributeError, ValueError, TypeError):
             bw, bh = ww, wh
         client_properties = window._client_properties
         resize_counter = window._resize_counter
@@ -1086,14 +1090,15 @@ class WindowClient(StubClientMixin):
 
     def _process_configure_override_redirect(self, packet):
         wid, x, y, w, h = packet[1:6]
-        window = self._id_to_window[wid]
+        window = self._id_to_window.get(wid)
         ax = self.sx(x)
         ay = self.sy(y)
         aw = max(1, self.sx(w))
         ah = max(1, self.sy(h))
         geomlog("_process_configure_override_redirect%s move resize window %s (id=%s) to %s",
                 packet[1:], window, wid, (ax,ay,aw,ah))
-        window.move_resize(ax, ay, aw, ah, -1)
+        if window:
+            window.move_resize(ax, ay, aw, ah, -1)
 
 
     def window_close_event(self, wid):
@@ -1196,10 +1201,18 @@ class WindowClient(StubClientMixin):
             if stdout_io_watch:
                 proc.stdout_io_watch = None
                 self.source_remove(stdout_io_watch)
+            stdout = proc.stdout
+            if stdout:
+                noerr(stdout.close)
+            stderr = proc.stderr
+            if stderr:
+                noerr(stderr.close)
             try:
-                proc.stdin.write(b"exit\n")
-                proc.stdin.flush()
-                proc.stdin.close()
+                stdin = proc.stdin
+                if stdin:
+                    stdin.write(b"exit\n")
+                    stdin.flush()
+                    stdin.close()
             except IOError:
                 log.warn("Warning: failed to tell the signal watcher to exit", exc_info=True)
             try:
@@ -1396,12 +1409,14 @@ class WindowClient(StubClientMixin):
         while self.exit_code is None:
             packet = self._draw_queue.get()
             if packet is None:
+                log("draw queue found exit marker")
                 break
             try:
                 self._do_draw(packet)
                 sleep(0)
             except Exception as e:
                 log.error("Error '%s' processing %s packet", e, packet[0], exc_info=True)
+        self._draw_thread = None
         log("draw thread ended")
 
     def _do_draw(self, packet):
@@ -1413,6 +1428,8 @@ class WindowClient(StubClientMixin):
                 window.eos()
             return
         x, y, width, height, coding, data, packet_sequence, rowstride = packet[2:10]
+        for v in (x, y, width, height, packet_sequence, rowstride):
+            assert isinstance(v, int), "expected int, found %s (%s)" % (v, type(v))
         coding = bytestostr(coding)
         if not window:
             #window is gone
@@ -1437,11 +1454,11 @@ class WindowClient(StubClientMixin):
         dtype = DRAW_TYPES.get(type(data), type(data))
         drawlog("process_draw: %7i %8s for window %3i, sequence %8i, %4ix%-4i at %4i,%-4i using %6s encoding with options=%s",
                 len(data), dtype, wid, packet_sequence, width, height, x, y, coding, options)
-        start = monotonic_time()
+        start = monotonic()
         def record_decode_time(success, message=""):
             if success>0:
-                end = monotonic_time()
-                decode_time = int(end*1000*1000-start*1000*1000)
+                end = monotonic()
+                decode_time = round(end*1000*1000-start*1000*1000)
                 self.pixel_counter.append((start, end, width*height))
                 dms = "%sms" % (int(decode_time/100)/10.0)
                 paintlog("record_decode_time(%s, %s) wid=%s, %s: %sx%s, %s",
@@ -1469,8 +1486,6 @@ class WindowClient(StubClientMixin):
         try:
             window.draw_region(x, y, width, height, coding, data, rowstride,
                                packet_sequence, options, [record_decode_time])
-        except KeyboardInterrupt:
-            raise
         except Exception as e:
             drawlog.error("Error drawing on window %i", wid, exc_info=True)
             self.idle_add(record_decode_time, False, str(e))
@@ -1487,10 +1502,10 @@ class WindowClient(StubClientMixin):
         return v
     def sx(self, v) -> int:
         """ convert X coordinate from server to client """
-        return iround(v)
+        return round(v)
     def sy(self, v) -> int:
         """ convert Y coordinate from server to client """
-        return iround(v)
+        return round(v)
     def srect(self, x, y, w, h):
         """ convert rectangle coordinates from server to client """
         return self.sx(x), self.sy(y), self.sx(w), self.sy(h)
@@ -1500,10 +1515,10 @@ class WindowClient(StubClientMixin):
 
     def cx(self, v) -> int:
         """ convert X coordinate from client to server """
-        return iround(v)
+        return round(v)
     def cy(self, v) -> int:
         """ convert Y coordinate from client to server """
-        return iround(v)
+        return round(v)
     def crect(self, x, y, w, h):
         """ convert rectangle coordinates from client to server """
         return self.cx(x), self.cy(y), self.cx(w), self.cy(h)
