@@ -7,11 +7,12 @@
 import os
 import struct
 import re
+from time import monotonic
 from io import BytesIO
 from gi.repository import GLib
 
 from xpra.net.compression import Compressible
-from xpra.os_util import POSIX, monotonic_time, strtobytes, bytestostr, hexstr, get_hex_uuid
+from xpra.os_util import POSIX, strtobytes, bytestostr, hexstr
 from xpra.util import csv, envint, envbool, repr_ellipsized, ellipsizer, typedict
 from xpra.platform.features import CLIPBOARDS as PLATFORM_CLIPBOARDS
 from xpra.log import Logger, is_debug_enabled
@@ -34,8 +35,6 @@ del CLIPBOARDS_ENV
 TEST_DROP_CLIPBOARD_REQUESTS = envint("XPRA_TEST_DROP_CLIPBOARD")
 DELAY_SEND_TOKEN = envint("XPRA_DELAY_SEND_TOKEN", 100)
 
-LOOP_DISABLE = envbool("XPRA_CLIPBOARD_LOOP_DISABLE", True)
-LOOP_PREFIX = os.environ.get("XPRA_CLIPBOARD_LOOP_PREFIX", "Xpra-Clipboard-Loop-Detection:")
 
 def get_discard_targets(envname="DISCARD", default_value=()):
     _discard_target_strs_ = os.environ.get("XPRA_%s_TARGETS" % envname)
@@ -105,8 +104,6 @@ class ClipboardProtocolHelperCore:
         self.max_clipboard_packet_size = d.intget("max-packet-size", MAX_CLIPBOARD_PACKET_SIZE)
         self.max_clipboard_receive_size = d.intget("max-receive-size", MAX_CLIPBOARD_RECEIVE_SIZE)
         self.max_clipboard_send_size = d.intget("max-send-size", MAX_CLIPBOARD_SEND_SIZE)
-        self.clipboard_contents_slice_fix = False
-        self.disabled_by_loop = []
         self.filter_res = []
         filter_res = d.strtupleget("filters")
         if filter_res:
@@ -124,8 +121,6 @@ class ClipboardProtocolHelperCore:
         self._want_targets = False
         self.init_packet_handlers()
         self.init_proxies(d.strtupleget("clipboards.local", CLIPBOARDS))
-        remote_loop_uuids = d.dictget("remote-loop-uuids", {})
-        self.verify_remote_loop_uuids(remote_loop_uuids)
         self.remote_clipboards = d.strtupleget("clipboards.remote", CLIPBOARDS)
 
     def init_translation(self, kwargs):
@@ -180,33 +175,13 @@ class ClipboardProtocolHelperCore:
 
     def cleanup(self):
         def nosend(*_args):
-            pass
+            """ during cleanup, stop sending packets """
         self.send = nosend
         for x in self._clipboard_proxies.values():
             x.cleanup()
         self._clipboard_proxies = {}
 
     def client_reset(self):
-        #if the client disconnects,
-        #we can re-enable the clipboards it had problems with:
-        l = self.disabled_by_loop
-        self.disabled_by_loop = []
-        for x in l:
-            proxy = self._clipboard_proxies.get(x)
-            proxy.set_enabled(True)
-
-
-    def get_loop_uuids(self):
-        uuids = {}
-        for proxy in self._clipboard_proxies.values():
-            uuids[proxy._selection] = proxy._loop_uuid
-        log("get_loop_uuids()=%s", uuids)
-        return uuids
-
-    def verify_remote_loop_uuids(self, uuids):
-        log("verify_remote_loop_uuids(%s)", uuids)
-
-    def _verify_remote_loop_uuids(self, clipboard, value, user_data):
         pass
 
     def set_direction(self, can_send, can_receive, max_send_size=None, max_receive_size=None):
@@ -222,16 +197,13 @@ class ClipboardProtocolHelperCore:
         if max_receive_size is not None:
             self.max_clipboard_receive_size = max_receive_size
 
-    def set_clipboard_contents_slice_fix(self, v):
-        self.clipboard_contents_slice_fix = v
-
-    def enable_selections(self, selections):
+    def enable_selections(self, selections=None):
         #when clients first connect or later through the "clipboard-enable-selections" packet,
         #they can tell us which clipboard selections they want enabled
         #(ie: OSX and win32 only use "CLIPBOARD" by default, and not "PRIMARY" or "SECONDARY")
         log("enabling selections: %s", csv(selections))
         for selection, proxy in self._clipboard_proxies.items():
-            proxy.set_enabled(bytestostr(selection) in selections)
+            proxy.set_enabled(selections and bytestostr(selection) in selections)
 
     def set_greedy_client(self, greedy):
         for proxy in self._clipboard_proxies.values():
@@ -254,7 +226,6 @@ class ClipboardProtocolHelperCore:
             "clipboard-contents-none"       : self._process_clipboard_contents_none,
             "clipboard-pending-requests"    : self._process_clipboard_pending_requests,
             "clipboard-enable-selections"   : self._process_clipboard_enable_selections,
-            "clipboard-loop-uuids"          : self._process_clipboard_loop_uuids,
             }
 
     def make_proxy(self, selection):
@@ -267,9 +238,9 @@ class ClipboardProtocolHelperCore:
             self._clipboard_proxies[selection] = proxy
         log("%s.init_proxies : %s", self, self._clipboard_proxies)
 
-    def init_proxies_uuid(self):
+    def init_proxies_claim(self):
         for proxy in self._clipboard_proxies.values():
-            proxy.init_uuid()
+            proxy.claim()
 
 
     # Used by the client during startup:
@@ -299,27 +270,25 @@ class ClipboardProtocolHelperCore:
             l("ignoring token for clipboard '%s' (no proxy)", name)
             return
         if not proxy.is_enabled():
-            l = log
-            if name not in self.disabled_by_loop:
-                l = log.warn
-            l("ignoring token for disabled clipboard '%s'", name)
+            log.warn("ignoring token for disabled clipboard '%s'", name)
             return
         log("process clipboard token selection=%s, local clipboard name=%s, proxy=%s", selection, name, proxy)
         targets = None
         target_data = None
         if proxy._can_receive:
             if len(packet)>=3:
-                targets = packet[2]
+                targets = self.local_targets(packet[2])
             if len(packet)>=8:
                 target, dtype, dformat, wire_encoding, wire_data = packet[3:8]
                 if target:
                     assert dformat in (8, 16, 32), "invalid format '%s' for datatype=%s and wire encoding=%s" % (
                         dformat, dtype, wire_encoding)
                     target = bytestostr(target)
-                    wire_encoding = bytestostr(wire_encoding)
-                    dtype = bytestostr(dtype)
-                    raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
-                    target_data = {target : (dtype, dformat, raw_data)}
+                    if not must_discard(target):
+                        wire_encoding = bytestostr(wire_encoding)
+                        dtype = bytestostr(dtype)
+                        raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
+                        target_data = {target : (dtype, dformat, raw_data)}
         #older versions always claimed the selection when the token is received:
         claim = True
         if len(packet)>=10:
@@ -331,6 +300,12 @@ class ClipboardProtocolHelperCore:
         synchronous_client = len(packet)>=11 and bool(packet[10])
         proxy.got_token(targets, target_data, claim, synchronous_client)
 
+    def local_targets(self, remote_targets):
+        return _filter_targets(remote_targets)
+
+    def remote_targets(self, local_targets):
+        return _filter_targets(local_targets)
+
     def _munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         log("_munge_raw_selection_to_wire%s", (target, dtype, dformat, repr_ellipsized(bytestostr(data))))
         # Some types just cannot be marshalled:
@@ -342,7 +317,7 @@ class ClipboardProtocolHelperCore:
             #targets is special cased here
             #because we can get the values in wire format already (not atoms)
             #thanks to the request_targets() function (required on win32)
-            return "atoms", _filter_targets(data)
+            return "atoms", self.remote_targets(data)
         try:
             return self._do_munge_raw_selection_to_wire(target, dtype, dformat, data)
         except Exception:
@@ -426,10 +401,7 @@ class ClipboardProtocolHelperCore:
             no_contents()
             return
         if not proxy.is_enabled():
-            l = log
-            if selection not in self.disabled_by_loop:
-                l = log.warn
-            l("Warning: ignoring clipboard request for '%s' (disabled)", name)
+            log.warn("Warning: ignoring clipboard request for '%s' (disabled)", name)
             no_contents()
             return
         if not proxy._can_send:
@@ -472,10 +444,7 @@ class ClipboardProtocolHelperCore:
         wire_data = self._may_compress(dtype, dformat, wire_data)
         if wire_data is not None:
             packet = ["clipboard-contents", request_id, selection,
-                    dtype, dformat, wire_encoding, wire_data]
-            if self.clipboard_contents_slice_fix:
-                #sending the extra argument requires the fix
-                packet.append(truncated)
+                    dtype, dformat, wire_encoding, wire_data, truncated]
             self.send(*packet)
 
     def _may_compress(self, dtype, dformat, wire_data):
@@ -483,7 +452,13 @@ class ClipboardProtocolHelperCore:
             log.warn("Warning: clipboard contents are too big and have not been sent")
             log.warn(" %s compressed bytes dropped (maximum is %s)", len(wire_data), self.max_clipboard_packet_size)
             return None
-        if isinstance(wire_data, (str, bytes)) and len(wire_data)>=MIN_CLIPBOARD_COMPRESS_SIZE:
+        l = len(wire_data)
+        if isinstance(wire_data, (str, bytes)) and l>=MIN_CLIPBOARD_COMPRESS_SIZE:
+            if isinstance(wire_data, str):
+                #compression requires bytes:
+                #but this would require the receiving end to use net_utf8()
+                wire_data = wire_data.encode("utf8")
+                log("encoded %i characters to %i utf8 bytes", l, len(wire_data))
             return Compressible("clipboard: %s / %s" % (dtype, dformat), wire_data)
         return wire_data
 
@@ -522,10 +497,6 @@ class ClipboardProtocolHelperCore:
         selections = tuple(bytestostr(x) for x in packet[1])
         self.enable_selections(selections)
 
-    def _process_clipboard_loop_uuids(self, packet):
-        loop_uuids = packet[1]
-        self.verify_remote_loop_uuids(loop_uuids)
-
 
     def process_clipboard_packet(self, packet):
         packet_type = bytestostr(packet[0])
@@ -551,7 +522,7 @@ class ClipboardProxyCore:
         self._greedy_client = False
         self._want_targets = False
         #semaphore to block the sending of the token when we change the owner ourselves:
-        self._block_owner_change = False
+        self._block_owner_change = 0
         self._last_emit_token = 0
         self._emit_token_timer = None
         #counters for info:
@@ -565,12 +536,6 @@ class ClipboardProxyCore:
         self._last_targets = ()
         self.preferred_targets = []
 
-        self._loop_uuid = ""
-
-    def init_uuid(self):
-        self._loop_uuid = LOOP_PREFIX+get_hex_uuid()
-        log("init_uuid() %s uuid=%s", self._selection, self._loop_uuid)
-
     def set_direction(self, can_send : bool, can_receive : bool):
         self._can_send = can_send
         self._can_receive = can_receive
@@ -587,7 +552,6 @@ class ClipboardProxyCore:
                 "preferred-targets"     : self.preferred_targets,
                 "blocked_owner_change"  : self._block_owner_change,
                 "last-targets"          : self._last_targets,
-                "loop-uuid"             : self._loop_uuid,
                 "event"         : {
                                    "selection_request"     : self._selection_request_events,
                                    "selection_get"         : self._selection_get_events,
@@ -603,6 +567,7 @@ class ClipboardProxyCore:
     def cleanup(self):
         self._enabled = False
         self.cancel_emit_token()
+        self.cancel_unblock()
 
     def is_enabled(self) -> bool:
         return self._enabled
@@ -634,35 +599,36 @@ class ClipboardProxyCore:
         if self._have_token or ((self._greedy_client or self._want_targets) and self._can_send):
             self.schedule_emit_token()
 
-    def schedule_emit_token(self):
-        if self._have_token or (not self._want_targets and not self._greedy_client) or DELAY_SEND_TOKEN<0:
+    def schedule_emit_token(self, min_delay=0):
+        if min_delay==0 and (self._have_token or (not self._want_targets and not self._greedy_client) or DELAY_SEND_TOKEN<0):
             #token ownership will change or told not to wait
             GLib.idle_add(self.emit_token)
         elif not self._emit_token_timer:
             #we already had sent the token,
             #or sending it is expensive, so wait a bit:
-            self.do_schedule_emit_token()
+            self.do_schedule_emit_token(min_delay)
 
-    def do_schedule_emit_token(self):
-        now = monotonic_time()
+    def do_schedule_emit_token(self, min_delay=0):
+        now = monotonic()
         elapsed = int((now-self._last_emit_token)*1000)
-        log("do_schedule_emit_token() selection=%s, elapsed=%i (max=%i)", self._selection, elapsed, DELAY_SEND_TOKEN)
-        if elapsed>=DELAY_SEND_TOKEN:
+        delay = max(min_delay, DELAY_SEND_TOKEN-elapsed)
+        log("do_schedule_emit_token(%i) selection=%s, elapsed=%i (max=%i), delay=%i",
+            min_delay, self._selection, elapsed, DELAY_SEND_TOKEN, delay)
+        if delay<=0:
             #enough time has passed
             self.emit_token()
         else:
-            self._emit_token_timer = GLib.timeout_add(DELAY_SEND_TOKEN-elapsed, self.emit_token)
+            self._emit_token_timer = GLib.timeout_add(delay, self.emit_token)
 
     def emit_token(self):
         self._emit_token_timer = None
         boc = self._block_owner_change
-        self._block_owner_change = True
+        if not boc:
+            self._block_owner_change = GLib.idle_add(self.remove_block)
         self._have_token = False
-        self._last_emit_token = monotonic_time()
+        self._last_emit_token = monotonic()
         self.do_emit_token()
         self._sent_token_events += 1
-        if boc is False:
-            GLib.idle_add(self.remove_block)
 
     def do_emit_token(self):
         #self.emit("send-clipboard-token")
@@ -674,6 +640,11 @@ class ClipboardProxyCore:
             self._emit_token_timer = None
             GLib.source_remove(ett)
 
+    def cancel_unblock(self):
+        boc = self._block_owner_change
+        if boc:
+            self._block_owner_change = 0
+            GLib.source_remove(boc)
 
     #def do_selection_request_event(self, event):
     #    pass
@@ -686,10 +657,13 @@ class ClipboardProxyCore:
 
     def remove_block(self, *_args):
         log("remove_block: %s", self._selection)
-        self._block_owner_change = False
+        self._block_owner_change = 0
 
     def claim(self):
-        pass
+        """
+        Subclasses may want to take ownership of the clipboard selection.
+        The X11 clipboard does.
+        """
 
     # This function is called by the xpra core when the peer has requested the
     # contents of this clipboard:
