@@ -9,41 +9,50 @@ import weakref
 
 #note: this is just for defining the order of encodings,
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
-PREFERED_ENCODING_ORDER = (
+PREFERRED_ENCODING_ORDER = (
     "h264", "vp9", "vp8", "mpeg4",
     "mpeg4+mp4", "h264+mp4", "vp8+webm", "vp9+webm",
     "png", "png/P", "png/L", "webp",
-    "rgb", "rgb24", "rgb32", "jpeg",
-    "h265", "jpeg2000", "mpeg1", "mpeg2",
+    "rgb", "rgb24", "rgb32", "jpeg", "jpega",
+    "h265", "mpeg1", "mpeg2",
+    "scroll",
+    "grayscale",
     )
 #encoding order for edges (usually one pixel high or wide):
 EDGE_ENCODING_ORDER = (
     "rgb24", "rgb32",
     "png", "webp",
-    "png/P", "png/L", "rgb", "jpeg",
+    "png/P", "png/L", "rgb", "jpeg", "jpega",
     )
 
 HELP_ORDER = (
-    "auto", "h264", "h265", "vp8", "vp9", "mpeg4",
+    "auto",
+    "grayscale",
+    "h264", "h265", "vp8", "vp9", "mpeg4",
     "png", "png/P", "png/L", "webp",
-    "rgb", "jpeg", "jpeg2000",
+    "rgb", "jpeg", "jpega",
+    "scroll",
     )
 
-#those are currently so useless that we don't want the user to select them by mistake
-PROBLEMATIC_ENCODINGS = ("h265", )
-
 
 #value: how much smaller the output is
 LOSSY_PIXEL_FORMATS = {
+    "NV12"    : 2,
     "YUV420P" : 2,
     "YUV422P" : 1.5,
     }
 
 PIXEL_SUBSAMPLING = {
+    "NV12"      : ((1, 1), (1, 2)),
     "YUV420P"   : ((1, 1), (2, 2), (2, 2)),
     "YUV422P"   : ((1, 1), (2, 1), (2, 1)),
     "YUV444P"   : ((1, 1), (1, 1), (1, 1)),
+    "YUV400P"   : ((1, 1), ),
     "GBRP"      : ((1, 1), (1, 1), (1, 1)),
+    "GBRP9LE"   : ((1, 1), (1, 1), (1, 1)),
+    "GBRP10"    : ((1, 1), (1, 1), (1, 1)),
+    "YUV444P10" : ((1, 1), (1, 1), (1, 1)),
+    "YUV444P16" : ((1, 1), (1, 1), (1, 1)),
 }
 def get_subsampling_divs(pixel_format):
     # Return size dividers for the given pixel format
@@ -71,7 +80,7 @@ class CodecStateException(Exception):
     pass
 
 
-class _codec_spec(object):
+class _codec_spec:
 
     #I can't imagine why someone would have more than this many
     #encoders or csc modules active at the same time!
@@ -119,10 +128,9 @@ class _codec_spec(object):
         log = Logger("encoding")
         cur = self.get_instance_count()
         if (self.max_instances>0 and cur>=self.max_instances) or cur>=_codec_spec.WARN_LIMIT:
+            instances = tuple(self.instances.keys())
             log.warn("Warning: already %s active instances of %s: %s",
-                     cur, self.codec_class, tuple(self.instances.keys()))
-            from xpra.util import dump_references
-            dump_references(log, self.instances.keys())
+                     cur, self.codec_class, instances)
         else:
             log("make_instance() %s - instance count=%s", self.codec_type, cur)
         v = self.codec_class()
@@ -130,16 +138,16 @@ class _codec_spec(object):
         return v
 
 
-    def get_instance_count(self):
+    def get_instance_count(self) -> int:
         return len(self.instances)
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         d = {}
         for k in self._exported_fields:
             d[k] = getattr(self, k)
         return d
 
-    def get_runtime_factor(self):
+    def get_runtime_factor(self) -> float:
         #a cost multiplier that some encoder may want to override
         #1.0 means no change:
         mi = self.max_instances
@@ -162,7 +170,7 @@ class video_spec(_codec_spec):
         self.input_colorspace = input_colorspace
         self.output_colorspaces = output_colorspaces    #ie: ["YUV420P" : "YUV420P", ...]
         self.has_lossless_mode = has_lossless_mode
-        _codec_spec.__init__(self, codec_class, codec_type, **kwargs)
+        super().__init__(codec_class, codec_type, **kwargs)
         self._exported_fields += ["encoding", "input_colorspace", "output_colorspaces", "has_lossless_mode"]
 
     def __repr__(self):
@@ -174,7 +182,7 @@ class csc_spec(_codec_spec):
     def __init__(self, input_colorspace, output_colorspace, codec_class, codec_type, **kwargs):
         self.input_colorspace = input_colorspace
         self.output_colorspace = output_colorspace
-        _codec_spec.__init__(self, codec_class, codec_type, **kwargs)
+        super().__init__(codec_class, codec_type, **kwargs)
         self._exported_fields += ["input_colorspace", "output_colorspace"]
 
     def __repr__(self):
