@@ -101,7 +101,7 @@ class StartSession(Gtk.Window):
         self.default_config = get_defaults()
         #log("default_config=%s", self.default_config)
         #log("options=%s (%s)", options, type(options))
-        Gtk.Window.__init__(self)
+        super().__init__()
         self.set_border_width(20)
         self.set_title("Start Xpra Session")
         self.set_position(Gtk.WindowPosition.CENTER)
@@ -286,8 +286,9 @@ class StartSession(Gtk.Window):
         vh.set_modules(video_decoders=self.session_options.video_decoders,
                        csc_modules=self.session_options.csc_modules or NO_GFX_CSC_OPTIONS)
         vh.init()
-        from xpra.codecs.loader import load_codecs  #pylint: disable=import-outside-toplevel
+        from xpra.codecs.loader import load_codecs, show_codecs  #pylint: disable=import-outside-toplevel
         load_codecs()
+        show_codecs()
         log("load_codecs() done")
 
 
@@ -678,7 +679,7 @@ class StartSession(Gtk.Window):
 
 class SessionOptions(Gtk.Window):
     def __init__(self, title, icon_name, options, run_mode, parent):
-        Gtk.Window.__init__(self)
+        super().__init__()
         self.options = options
         self.run_mode = run_mode
         self.set_title(title)
@@ -719,6 +720,19 @@ class SessionOptions(Gtk.Window):
         tb.attach(al, 0, 2)
         tb.inc()
 
+    def _save_widget(self, fn, widget, widget_type, **kwargs):
+        setattr(self, "%s_widget" % fn, widget)
+        setattr(self, "%s_widget_type" % fn, widget_type)
+        for k,v in kwargs.items():
+            setattr(self, "%s_%s" % (fn, k), v)
+
+    def get_widget(self, fn):
+        return getattr(self, "%s_widget" % fn)
+
+    def get_widget_options(self, fn):
+        return getattr(self, "%s_options" % fn)
+
+
     def bool_cb(self, table, label, option_name, tooltip_text=None, link=None):
         attach_label(table, label, tooltip_text, link)
         fn = option_name.replace("-", "_")
@@ -728,9 +742,7 @@ class SessionOptions(Gtk.Window):
         cb.set_active(active)
         al = xal(cb, xalign=0)
         table.attach(al, 1)
-        setattr(self, "%s_widget" % fn, cb)
-        setattr(self, "%s_widget_type" % fn, "bool")
-        setattr(self, "%s_values" % fn, [value if not active else False, value if active else True])
+        self._save_widget(fn, cb, "bool", values=[value if not active else False, value if active else True])
         self.widgets.append(option_name)
         table.inc()
         return cb
@@ -746,7 +758,6 @@ class SessionOptions(Gtk.Window):
         attach_label(table, label, tooltip_text, link)
         fn = option_name.replace("-", "_")
         widget_base_name = "%s_widget" % fn
-        setattr(self, "%s_widget_type" % fn, "radio")
         self.widgets.append(option_name)
         value = getattr(self.options, fn)
         #log.warn("%s=%s", fn, value)
@@ -771,7 +782,7 @@ class SessionOptions(Gtk.Window):
             i += 1
             saved_options[label] = saved_match
             btns.append(btn)
-        setattr(self, "%s_options" % fn, saved_options)
+        self._save_widget(fn, btns, "radio", options=saved_options)
         table.attach(hbox, 1)
         table.inc()
         return btns
@@ -791,9 +802,7 @@ class SessionOptions(Gtk.Window):
         al = Gtk.Alignment(xalign=0, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(c)
         table.attach(al, 1)
-        setattr(self, "%s_widget" % fn, c)
-        setattr(self, "%s_widget_type" % fn, "combo")
-        setattr(self, "%s_options" % fn, options)
+        self._save_widget(fn, c, "combo", options=options)
         self.widgets.append(option_name)
         table.inc()
         return c
@@ -814,8 +823,7 @@ class SessionOptions(Gtk.Window):
             for v,label in marks.items():
                 c.add_mark(v, Gtk.PositionType.BOTTOM, label)
         table.attach(c, 1)
-        setattr(self, "%s_widget" % fn, c)
-        setattr(self, "%s_widget_type" % fn, "scale")
+        self._save_widget(fn, c, "scale")
         self.widgets.append(option_name)
         table.inc()
         return c
@@ -834,8 +842,7 @@ class SessionOptions(Gtk.Window):
         elif widget_type=="combo":
             values = self.valuesfromcombo(option_name)
         elif widget_type=="scale":
-            widget = getattr(self, "%s_widget" % fn)
-            values = (int(widget.get_value()), )
+            values = self.valuesfromscale(option_name)
         else:
             log.warn("unknown widget type '%s'", widget_type)
         if len(values)!=1 or values[0]!=UNSET:
@@ -851,14 +858,14 @@ class SessionOptions(Gtk.Window):
 
     def valuesfromswitch(self, option_name):
         fn = option_name.replace("-", "_")
-        widget = getattr(self, "%s_widget" % fn)
+        widget = self.get_widget(fn)
         values = getattr(self, "%s_values" % fn)
         value = values[int(widget.get_active())]
         return (value, )
 
     def valuesfromradio(self, option_name):
         fn = option_name.replace("-", "_")
-        options = getattr(self, "%s_options" % fn)
+        options = self.get_widget_options(fn)
         widget_base_name = "%s_widget" % fn
         for label, match in options.items():
             btn = getattr(self, "%s_%s" % (widget_base_name, label))
@@ -868,14 +875,19 @@ class SessionOptions(Gtk.Window):
 
     def valuesfromcombo(self, option_name):
         fn = option_name.replace("-", "_")
-        widget = getattr(self, "%s_widget" % fn)
-        options = getattr(self, "%s_options" % fn)
+        widget = self.get_widget(fn)
+        options = self.get_widget_options(fn)
         value = widget.get_active_text()
         for k,v in options.items():
             if str(v)==value:
                 return (k, )
         return (UNSET, )
 
+    def valuesfromscale(self, option_name):
+        fn = option_name.replace("-", "_")
+        widget = self.get_widget(fn)
+        return (int(widget.get_value()), )
+
     def table(self):
         tb = TableBuilder()
         table = tb.get_table()
