@@ -14,7 +14,7 @@ LOG_PREFIX = os.environ.get("XPRA_LOG_PREFIX", "")
 LOG_FORMAT = os.environ.get("XPRA_LOG_FORMAT", "%(asctime)s %(message)s")
 NOPREFIX_FORMAT = "%(message)s"
 
-DEBUG_MODULES = os.environ.get("XPRA_DEBUG_MODULES", "").split(",")
+DEBUG_MODULES = [x for x in os.environ.get("XPRA_DEBUG_MODULES", "").split(",") if x]
 
 logging.basicConfig(format=LOG_FORMAT)
 logging.root.setLevel(logging.INFO)
@@ -56,7 +56,7 @@ def get_debug_args():
     return args
 
 class FullDebugContext:
-
+    __slots__ = ("debug_enabled_categories", "enabled")
     def __enter__(self):
         global debug_enabled_categories
         self.debug_enabled_categories = debug_enabled_categories
@@ -247,6 +247,7 @@ STRUCT_KNOWN_FILTERS = {
                 "libav"         : "libav common code (used by swscale, avcodec and ffmpeg)",
                 "ffmpeg"        : "ffmpeg encoder",
                 "pillow"        : "Pillow encoder and decoder",
+                "spng"          : "spng codec",
                 "jpeg"          : "JPEG codec",
                 "vpx"           : "libvpx encoder and decoder",
                 "nvjpeg"        : "nvidia nvjpeg hardware encoder",
@@ -283,7 +284,6 @@ STRUCT_KNOWN_FILTERS = {
                 "protocol"      : "Packet input and output (formatting, parsing, sending and receiving)",
                 "websocket"     : "WebSocket layer",
                 "named-pipe"    : "Named pipe",
-                "udp"           : "UDP",
                 "crypto"        : "Encryption",
                 "auth"          : "Authentication",
                 "upnp"           : "UPnP",
@@ -335,23 +335,41 @@ for d in STRUCT_KNOWN_FILTERS.values():
 def isenvdebug(category : str) -> bool:
     return os.environ.get("XPRA_%s_DEBUG" % category.upper().replace("-", "_").replace("+", "_"), "0")=="1"
 
-# A wrapper around 'logging' with some convenience stuff.  In particular:
-#    -- You initialize it with a list of categories
-#       If unset, the default logging target is set to the name of the module where
-#       Logger() was called.
-#    -- Any of the categories can enable debug logging if the environment
-#       variable 'XPRA_${CATEGORY}_DEBUG' is set to "1"
-#    -- We also keep a list of debug_categories, so these can get enabled
-#       programatically too
-#    -- We keep track of which loggers are associated with each category,
-#       so we can enable/disable debug logging by category
-#    -- You can pass exc_info=True to any method, and sys.exc_info() will be
-#       substituted.
-#    -- __call__ is an alias for debug
-#    -- we bypass the logging system unless debugging is enabled for the logger,
-#       which is much faster than relying on the python logging code
+
+def get_info():
+    return {
+        "filters" : STRUCT_KNOWN_FILTERS,
+        "categories" : {
+            "enabled"   : tuple(debug_enabled_categories),
+            "disabled"  : tuple(debug_disabled_categories),
+            },
+        "handler"   : getattr(global_logging_handler, "__name__", "<unknown>"),
+        "prefix"    : LOG_PREFIX,
+        "format"    : LOG_FORMAT,
+        "debug-modules" : DEBUG_MODULES,
+        #all_loggers
+        }
+
 
 class Logger:
+    """
+    A wrapper around 'logging' with some convenience stuff.  In particular:
+    * You initialize it with a list of categories
+        If unset, the default logging target is set to the name of the module where
+        Logger() was called.
+    * Any of the categories can enable debug logging if the environment
+    variable 'XPRA_${CATEGORY}_DEBUG' is set to "1"
+    * We also keep a list of debug_categories, so these can get enabled
+        programatically too
+    * We keep track of which loggers are associated with each category,
+        so we can enable/disable debug logging by category
+    * You can pass exc_info=True to any method, and sys.exc_info() will be
+        substituted.
+    * __call__ is an alias for debug
+    * we bypass the logging system unless debugging is enabled for the logger,
+        which is much faster than relying on the python logging code
+    """
+    __slots__ = ("categories", "level_override", "logger", "debug_enabled", "__weakref__")
     def __init__(self, *categories):
         global default_level, debug_disabled_categories, KNOWN_FILTERS
         self.categories = list(categories)
@@ -390,6 +408,8 @@ class Logger:
         for x in categories:
             if x not in KNOWN_FILTERS:
                 self.warn("unknown logging category: %s", x)
+        if self.debug_enabled:
+            self.debug("debug enabled for %s / %s", caller, categories)
 
     def get_info(self) -> dict:
         return {
@@ -438,7 +458,6 @@ class Logger:
 
 
 class CaptureHandler(logging.Handler):
-
     def __init__(self):
         super().__init__(logging.DEBUG)
         self.records = []
