@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -24,7 +24,7 @@ def init():
         do_init()
 
 def do_init():
-    pass
+    """ some platforms override this """
 
 _ready_done = False
 def ready():
@@ -34,7 +34,7 @@ def ready():
         do_ready()
 
 def do_ready():
-    pass
+    """ some platforms override this """
 
 
 _default_icon = "xpra.png"
@@ -94,8 +94,7 @@ def get_icon_size():
     xdpi = get_xdpi()
     ydpi = get_ydpi()
     if xdpi>0 and ydpi>0:
-        from xpra.util import iround
-        dpi = iround((xdpi + ydpi)/2.0)
+        dpi = round((xdpi + ydpi)/2)
     else:
         dpi = 96
     if dpi > 144:
@@ -133,7 +132,11 @@ def get_pillow_icc_info():
         from PIL.ImageCms import get_display_profile
         INTENT_STR = {}
         for x in ("PERCEPTUAL", "RELATIVE_COLORIMETRIC", "SATURATION", "ABSOLUTE_COLORIMETRIC"):
-            v = getattr(ImageCms, "INTENT_%s" % x, None)
+            intent = getattr(ImageCms, "Intent", None)
+            if intent:
+                v = getattr(intent, x, None)
+            else:
+                v = getattr(ImageCms, "INTENT_%s" % x, None)
             if v:
                 INTENT_STR[v] = x.lower().replace("_", "-")
         screenlog("get_icc_info() intents=%s", INTENT_STR)
@@ -219,20 +222,29 @@ def get_window_frame_sizes():
 
 
 def add_window_hooks(_window):
-    pass
+    """
+    To add platform specific code to each window,
+    called when the window is created.
+    """
 
 def remove_window_hooks(_window):
-    pass
+    """
+    Remove the hooks,
+    called when the window is destroyed
+    """
 
 
 def show_desktop(_show):
-    pass
+    """ If possible, show the desktop """
 
 def set_fullscreen_monitors(_window, _fsm, _source_indication=0):
-    pass
+    """ Only overriden by xposix """
 
 def set_shaded(_window, _shaded):
-    pass
+    """
+    GTK never exposed the 'shaded' window attribute,
+    xposix clients will hook it up here.
+    """
 
 
 def gl_check():
