@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,8 +9,7 @@ from xpra.util import typedict
 
 
 class Authenticator:
-    def __init__(self, username, **kwargs):
-        self.username = username
+    def __init__(self, **kwargs):
         self.challenge_sent = False
         self.prompt = kwargs.pop("prompt", "password")
         self.passed = False
