@@ -7,30 +7,32 @@
 
 import shlex
 import os.path
+from time import monotonic
 from subprocess import Popen, PIPE
 from threading import Event
 from gi.repository import GLib
 
-from xpra.os_util import pollwait, monotonic_time, bytestostr, osexpand, OSX, POSIX
+from xpra.os_util import pollwait, bytestostr, osexpand, OSX, POSIX
 from xpra.util import typedict, envbool, csv, engs
 from xpra.make_thread import start_thread
-from xpra.platform import get_username
+from xpra.platform.info import get_username
 from xpra.platform.paths import get_icon_filename
 from xpra.scripts.parsing import sound_option
+from xpra.scripts.server import save_session_file
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger
 
 log = Logger("server")
 soundlog = Logger("sound")
-httplog = Logger("http")
+httplog = Logger("sound", "http")
 
 PRIVATE_PULSEAUDIO = envbool("XPRA_PRIVATE_PULSEAUDIO", True)
 
 
-"""
-Mixin for servers that handle audio forwarding.
-"""
 class AudioServer(StubServerMixin):
+    """
+    Mixin for servers that handle audio forwarding.
+    """
 
     def __init__(self):
         self.audio_init_done = Event()
@@ -108,102 +110,7 @@ class AudioServer(StubServerMixin):
 
 
     def get_http_scripts(self) -> dict:
-        return {
-            "/audio.mp3" : self.http_audio_mp3_request,
-            }
-
-    def http_audio_mp3_request(self, handler):
-        def err(code=500):
-            handler.send_response(code)
-            return None
-        from xpra.net.http_handler import parse_url
-        args = parse_url(handler)
-        if not args:
-            return err()
-        httplog("http_audio_mp3_request(%s) args=%s", handler, args)
-        uuid = args.get("uuid")
-        if not uuid:
-            httplog.warn("Warning: http-stream audio request, missing uuid")
-            return err()
-        source = None
-        for x in self._server_sources.values():
-            if x.uuid==uuid:
-                source = x
-                break
-        if not source:
-            httplog.warn("Warning: no client matching uuid '%s'", uuid)
-            return err()
-        #don't close the connection when handler.finish() is called,
-        #we will continue to write to this socket as we process more buffers:
-        finish = handler.finish
-        def do_finish():
-            try:
-                finish()
-            except Exception:
-                log("error calling %s", finish, exc_info=True)
-        def noop():
-            pass
-        handler.finish = noop
-        state = {}
-        def new_buffer(_sound_source, data, _metadata, packet_metadata=()):
-            if state.get("failed"):
-                return
-            if not state.get("started"):
-                httplog.warn("buffer received but stream is not started yet")
-                source.stop_sending_sound()
-                err()
-                do_finish()
-                return
-            count = state.get("buffers", 0)
-            httplog("new_buffer [%i] for %s sound stream: %i bytes", count, state.get("codec", "?"), len(data))
-            #httplog("buffer %i: %s", count, hexstr(data))
-            state["buffers"] = count+1
-            try:
-                for x in packet_metadata:
-                    handler.wfile.write(x)
-                handler.wfile.write(data)
-                handler.wfile.flush()
-            except Exception as e:
-                state["failed"] = True
-                httplog("failed to send new audio buffer", exc_info=True)
-                httplog.warn("Error: failed to send audio packet:")
-                httplog.warn(" %s", e)
-                source.stop_sending_sound()
-                do_finish()
-        def new_stream(sound_source, codec):
-            codec = bytestostr(codec)
-            httplog("new_stream: %s", codec)
-            sound_source.codec = codec
-            headers = {
-                "Content-type"      : "audio/mpeg",
-                }
-            try:
-                handler.send_response(200)
-                for k,v in headers.items():
-                    handler.send_header(k, v)
-                handler.end_headers()
-            except ValueError:
-                httplog("new_stream error writing headers", exc_info=True)
-                state["failed"] = True
-                source.stop_sending_sound()
-                do_finish()
-            else:
-                state["started"] = True
-                state["buffers"] = 0
-                state["codec"] = codec
-        start = monotonic_time()
-        def timeout_check():
-            self.http_stream_check_timers.pop(start, None)
-            if not state.get("started"):
-                err()
-                source.stop_sending_sound()
-        if source.sound_source:
-            source.stop_sending_sound()
-        def start_sending_sound():
-            source.start_sending_sound("mp3", volume=1.0, new_stream=new_stream,
-                                       new_buffer=new_buffer, skip_client_codec_check=True)
-        GLib.idle_add(start_sending_sound)
-        self.http_stream_check_timers[start] = (self.timeout_add(1000*5, timeout_check), source.stop_sending_sound)
+        return {}
 
 
     def init_pulseaudio(self):
@@ -242,10 +149,16 @@ class AudioServer(StubServerMixin):
                 log.warn("Warning: the runtime directory '%s' does not exist,", rd)
                 log.warn(" cannot start a private pulseaudio server")
             else:
-                xpra_rd = _get_xpra_runtime_dir()
+                xpra_rd = os.environ.get("XPRA_SESSION_DIR", _get_xpra_runtime_dir())
                 assert xpra_rd, "bug: no xpra runtime dir"
                 display = os.environ.get("DISPLAY", "").lstrip(":")
-                self.pulseaudio_private_dir = osexpand(os.path.join(xpra_rd, "pulse-%s" % display))
+                if xpra_rd.find("/%s/" % display):
+                    #this is already a per-display directory,
+                    #no need to include the display name again:
+                    pulse_dirname = "pulse"
+                else:
+                    pulse_dirname = "pulse-%s" % display
+                self.pulseaudio_private_dir = osexpand(os.path.join(xpra_rd, pulse_dirname))
                 if not os.path.exists(self.pulseaudio_private_dir):
                     os.mkdir(self.pulseaudio_private_dir, 0o700)
                 env["XDG_RUNTIME_DIR"] = self.pulseaudio_private_dir
@@ -271,7 +184,7 @@ class AudioServer(StubServerMixin):
                 self.clean_pulseaudio_private_dir()
                 return
             cmd[0] = pa_cmd
-        started_at = monotonic_time()
+        started_at = monotonic()
         def pulseaudio_warning():
             soundlog.warn("Warning: pulseaudio has terminated shortly after startup.")
             soundlog.warn(" pulseaudio is limited to a single instance per user account,")
@@ -284,7 +197,7 @@ class AudioServer(StubServerMixin):
             if self.pulseaudio_proc is None or self._closing:
                 #cleared by cleanup already, ignore
                 return
-            elapsed = monotonic_time()-started_at
+            elapsed = monotonic()-started_at
             if elapsed<2:
                 self.timeout_add(1000, pulseaudio_warning)
             else:
@@ -302,6 +215,7 @@ class AudioServer(StubServerMixin):
             return
         self.add_process(self.pulseaudio_proc, "pulseaudio", cmd, ignore=True, callback=pulseaudio_ended)
         if self.pulseaudio_proc:
+            save_session_file("pulseaudio.pid", "%s" % self.pulseaudio_proc.pid)
             soundlog.info("pulseaudio server started with pid %s", self.pulseaudio_proc.pid)
             if self.pulseaudio_private_socket:
                 soundlog.info(" private server socket path:")
@@ -349,10 +263,12 @@ class AudioServer(StubServerMixin):
             #wait for the socket to get cleaned up
             #(it should be removed by the pulseaudio server as it exits)
             import time
-            now = monotonic_time()
-            while (monotonic_time()-now)<1 and os.path.exists(self.pulseaudio_private_socket):
+            now = monotonic()
+            while (monotonic()-now)<1 and os.path.exists(self.pulseaudio_private_socket):
                 time.sleep(0.1)
         self.clean_pulseaudio_private_dir()
+        if not self.is_child_alive(proc):
+            self.do_clean_session_files("pulseaudio.pid")
 
     def clean_pulseaudio_private_dir(self):
         if self.pulseaudio_private_dir:
