@@ -363,7 +363,7 @@ class XpraDesktopServer(DesktopServerBaseClass):
         self.do_modify_gsettings(self.gsettings_modified, True)
 
     def notify_dpi_warning(self, body):
-        pass
+        """ ignore DPI warnings in desktop mode """
 
     def print_screen_info(self):
         super().print_screen_info()
@@ -637,8 +637,6 @@ class XpraDesktopServer(DesktopServerBaseClass):
 
     def _process_desktop_size(self, proto, packet):
         pass
-    def calculate_desktops(self):
-        pass
     def calculate_workarea(self, w, h):
         pass
 
