@@ -1,27 +1,24 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+from gi.repository import GObject, Gdk
 
 from xpra.util import envbool
-from xpra.os_util import PYTHON2
+from xpra.common import MAX_WINDOW_SIZE
 from xpra.gtk_common.error import xsync, xswallow
-from xpra.x11.gtk_x11.prop import prop_set, prop_get
+from xpra.x11.gtk_x11.prop import prop_set, prop_get, prop_del
 from xpra.x11.window_info import window_name, window_info
 from xpra.gtk_common.gobject_util import no_arg_signal, one_arg_signal
-from xpra.gtk_common.gtk_util import (
-    get_default_root_window, display_get_default, get_xwindow,
-    GDKWINDOW_FOREIGN, CLASS_INPUT_ONLY, GDKWindow,
-    PARAM_READABLE,
-    )
-from xpra.x11.common import Unmanageable, MAX_WINDOW_SIZE
+from xpra.gtk_common.gtk_util import get_default_root_window
+from xpra.x11.common import Unmanageable
+from xpra.x11.gtk_x11 import GDKX11Window
 from xpra.x11.gtk_x11.selection import ManagerSelection
-from xpra.x11.gtk_x11.world_window import WorldWindow
 from xpra.x11.models.window import WindowModel, configure_bits
-from xpra.x11.gtk_x11.world_window import WorldWindow
+from xpra.x11.gtk_x11.world_window import WorldWindow, destroy_world_window
 from xpra.x11.gtk_x11.gdk_bindings import (
     add_event_receiver,                              #@UnresolvedImport
     add_fallback_receiver, remove_fallback_receiver, #@UnresolvedImport
@@ -29,13 +26,10 @@ from xpra.x11.gtk_x11.gdk_bindings import (
     )
 from xpra.x11.bindings.window_bindings import constants, X11WindowBindings #@UnresolvedImport
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
-from xpra.gtk_common.gobject_compat import import_gobject, is_gtk3
 from xpra.log import Logger
 
 log = Logger("x11", "window")
 
-gobject = import_gobject()
-
 X11Window = X11WindowBindings()
 X11Keyboard = X11KeyboardBindings()
 
@@ -144,8 +138,9 @@ DEFAULT_NET_SUPPORTED = [
         "_MOTIF_WM_INFO",
 
         "_NET_REQUEST_FRAME_EXTENTS",
-        # Not at all yet:
-        #"_NET_RESTACK_WINDOW",
+        "_NET_RESTACK_WINDOW",
+
+        "_NET_WM_OPAQUE_REGION",
         ]
 FRAME_EXTENTS = envbool("XPRA_FRAME_EXTENTS", True)
 if FRAME_EXTENTS:
@@ -153,23 +148,18 @@ if FRAME_EXTENTS:
 
 NET_SUPPORTED = [x for x in DEFAULT_NET_SUPPORTED if x not in NO_NET_SUPPORTED]
 
-def u(s):
-    if PYTHON2:
-        return s.decode("utf8")
-    return s
-
 DEFAULT_SIZE_CONSTRAINTS = (0, 0, MAX_WINDOW_SIZE, MAX_WINDOW_SIZE)
 
 
-class Wm(gobject.GObject):
+class Wm(GObject.GObject):
 
     __gproperties__ = {
-        "windows": (gobject.TYPE_PYOBJECT,
+        "windows": (GObject.TYPE_PYOBJECT,
                     "Set of managed windows (as WindowModels)", "",
-                    PARAM_READABLE),
-        "toplevel": (gobject.TYPE_PYOBJECT,
+                    GObject.ParamFlags.READABLE),
+        "toplevel": (GObject.TYPE_PYOBJECT,
                      "Toplevel container widget for the display", "",
-                     PARAM_READABLE),
+                     GObject.ParamFlags.READABLE),
         }
     __gsignals__ = {
         # Public use:
@@ -191,10 +181,10 @@ class Wm(gobject.GObject):
         }
 
     def __init__(self, replace_other_wm, wm_name, display=None):
-        gobject.GObject.__init__(self)
+        super().__init__()
 
         if display is None:
-            display = display_get_default()
+            display = Gdk.Display.get_default()
         self._display = display
         self._root = self._display.get_default_screen().get_root_window()
         self._wm_name = wm_name
@@ -236,10 +226,6 @@ class Wm(gobject.GObject):
         self._world_window = WorldWindow(self._display.get_default_screen())
         self.notify("toplevel")
         self._world_window.show_all()
-        if not is_gtk3():
-            self._world_window = WorldWindow(self._display.get_default_screen())
-            self.notify("toplevel")
-            self._world_window.show_all()
 
         # Okay, ready to select for SubstructureRedirect and then load in all
         # the existing clients.
@@ -249,15 +235,15 @@ class Wm(gobject.GObject):
         #to a window that is already destroyed
         #and we don't want to miss those events, so:
         add_fallback_receiver("child-map-request-event", self)
-        rxid = get_xwindow(self._root)
+        rxid = self._root.get_xid()
         X11Window.substructureRedirect(rxid)
 
         for w in get_children(self._root):
             # Checking for FOREIGN here filters out anything that we've
             # created ourselves (like, say, the world window), and checking
             # for mapped filters out any withdrawn windows.
-            xid = get_xwindow(w)
-            if (w.get_window_type() == GDKWINDOW_FOREIGN
+            xid = w.get_xid()
+            if (w.get_window_type() == Gdk.WindowType.FOREIGN
                 and not X11Window.is_override_redirect(xid)
                 and X11Window.is_mapped(xid)):
                 log("Wm managing pre-existing child window %#x", xid)
@@ -345,7 +331,7 @@ class Wm(gobject.GObject):
                 l = log.warn
             else:
                 l = log
-            l("Warning: failed to manage client window %#x:", get_xwindow(gdkwindow))
+            l("Warning: failed to manage client window %#x:", gdkwindow.get_xid())
             l(" %s", e)
             l("", exc_info=True)
             with xswallow:
@@ -385,7 +371,6 @@ class Wm(gobject.GObject):
         #   _NET_CURRENT_DESKTOP
         #   _NET_WM_PING responses
         # and maybe:
-        #   _NET_RESTACK_WINDOW
         #   _NET_WM_STATE
         log("do_xpra_client_message_event(%s)", event)
         if event.message_type=="_NET_SHOWING_DESKTOP":
@@ -397,14 +382,14 @@ class Wm(gobject.GObject):
             #so this must be a an unmapped window
             frame = (0, 0, 0, 0)
             with xswallow:
-                if not X11Window.is_override_redirect(get_xwindow(event.window)):
+                if not X11Window.is_override_redirect(event.window.get_xid()):
                     #use the global default:
                     frame = prop_get(self._root, "DEFAULT_NET_FRAME_EXTENTS", ["u32"], ignore_errors=True)
                 if not frame:
                     #fallback:
                     frame = (0, 0, 0, 0)
                 framelog("_NET_REQUEST_FRAME_EXTENTS: setting _NET_FRAME_EXTENTS=%s on %#x",
-                         frame, get_xwindow(event.window))
+                         frame, event.window.get_xid())
                 prop_set(event.window, "_NET_FRAME_EXTENTS", ["u32"], frame)
 
     def _lost_wm_selection(self, selection):
@@ -419,6 +404,11 @@ class Wm(gobject.GObject):
         remove_fallback_receiver("child-map-request-event", self)
         for win in tuple(self._windows.values()):
             win.unmanage(True)
+        with xswallow:
+            prop_del(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK")
+            prop_del(self._ewmh_window, "_NET_WM_NAME")
+        destroy_world_window()
+
 
     def do_child_map_request_event(self, event):
         log("Found a potential client")
@@ -444,7 +434,7 @@ class Wm(gobject.GObject):
         log("do_child_configure_request_event(%s) value_mask=%s, reconfigure on withdrawn window",
             event, configure_bits(event.value_mask))
         with xswallow:
-            xid = get_xwindow(event.window)
+            xid = event.window.get_xid()
             x, y, w, h = X11Window.getGeometry(xid)[:4]
             if event.value_mask & CWX:
                 x = event.x
@@ -474,7 +464,7 @@ class Wm(gobject.GObject):
     def set_desktop_list(self, desktops):
         log("set_desktop_list(%s)", desktops)
         self.root_set("_NET_NUMBER_OF_DESKTOPS", "u32", len(desktops))
-        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], [u(d) for d in desktops])
+        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], desktops)
 
     def set_current_desktop(self, index):
         self.root_set("_NET_CURRENT_DESKTOP", "u32", index)
@@ -492,11 +482,11 @@ class Wm(gobject.GObject):
         # clobber any XSelectInput calls that *we* might have wanted to make
         # on this window.)  Also, GDK might silently swallow all events that
         # are detected on it, anyway.
-        self._ewmh_window = GDKWindow(self._root, wclass=CLASS_INPUT_ONLY, title=self._wm_name)
+        self._ewmh_window = GDKX11Window(self._root, wclass=Gdk.WindowWindowClass.INPUT_ONLY, title=self._wm_name)
         prop_set(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK",
                  "window", self._ewmh_window)
         self.root_set("_NET_SUPPORTING_WM_CHECK", "window", self._ewmh_window)
-        self.root_set("_NET_WM_NAME", "utf8", u(self._wm_name))
+        self.root_set("_NET_WM_NAME", "utf8", self._wm_name)
 
     def get_net_wm_name(self):
         try:
@@ -504,4 +494,4 @@ class Wm(gobject.GObject):
         except Exception as e:
             log.error("error querying _NET_WM_NAME: %s", e)
 
-gobject.type_register(Wm)
+GObject.type_register(Wm)
