@@ -9,9 +9,10 @@
 import os
 
 from xpra.os_util import bytestostr, strtobytes, hexstr
-from xpra.util import typedict, envbool, iround
+from xpra.util import typedict, envbool
 from xpra.gtk_common.error import xswallow, xsync, xlog
 from xpra.scripts.config import parse_bool
+from xpra.server import EXITING_CODE
 from xpra.x11.x11_server_core import X11ServerCore, XTestPointerDevice
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.x11.xsettings_prop import XSettingsTypeInteger, XSettingsTypeString, BLACKLISTED_XSETTINGS
@@ -28,6 +29,11 @@ SCALED_FONT_ANTIALIAS = envbool("XPRA_SCALED_FONT_ANTIALIAS", False)
 SYNC_ICC = envbool("XPRA_SYNC_ICC", True)
 
 
+def root_prop_set(prop_name, prop_type, value):
+    from xpra.gtk_common.gtk_util import get_default_root_window
+    from xpra.x11.gtk_x11.prop import prop_set
+    prop_set(get_default_root_window(), prop_name, prop_type, value)
+
 def _get_antialias_hintstyle(antialias):
     hintstyle = antialias.strget("hintstyle", "").lower()
     if hintstyle in ("hintnone", "hintslight", "hintmedium", "hintfull"):
@@ -78,44 +84,45 @@ class X11ServerBase(X11ServerCore):
             self.init_all_server_settings()
 
     def save_pid(self):
-        from xpra.scripts.server import _save_int
-        _save_int(b"XPRA_SERVER_gPID", os.getpid())
+        root_prop_set(b"XPRA_SERVER_PID", "u32", os.getpid())
+
+    def clean_x11_properties(self):
+        super().clean_x11_properties()
+        self.do_clean_x11_properties("XPRA_SERVER_PID")
+
 
     def init_display_pid(self, pid):
-        if pid:
-            from xpra.scripts.server import _save_int
-            _save_int(b"XPRA_XVFB_PID", pid)
+        if not pid:
+            log.info("xvfb pid not found")
         else:
-            from xpra.scripts.server import _get_int
-            pid = _get_int(b"XPRA_XVFB_PID") or _get_int(b"_XPRA_SERVER_PID")
-            if not pid:
-                log.info("xvfb pid not found")
-            else:
-                log.info("xvfb pid=%i", pid)
+            log.info("xvfb pid=%i", pid)
         self.display_pid = pid
 
     def kill_display(self):
         if self.display_pid:
-            from xpra.server import EXITING_CODE
             if self._upgrading==EXITING_CODE:
                 log.info("exiting: not cleaning up Xvfb")
             elif self._upgrading:
                 log.info("upgrading: not cleaning up Xvfb")
             else:
-                from xpra.scripts.server import add_cleanup
                 from xpra.x11.vfb_util import kill_xvfb
-                def do_kill_display():
-                    kill_xvfb(self.display_pid)
-                add_cleanup(do_kill_display)
+                kill_xvfb(self.display_pid)
+                self.do_clean_session_files(
+                    "xvfb.pid",
+                    "xauthority",
+                    "Xorg.log*",
+                    "xorg.conf.d/*"
+                    "xorg.conf.d"
+                    )
 
 
-    def do_cleanup(self):
-        X11ServerCore.do_cleanup(self)
+    def late_cleanup(self):
+        super().late_cleanup()
         self.kill_display()
 
 
     def configure_best_screen_size(self):
-        root_w, root_h = X11ServerCore.configure_best_screen_size(self)
+        root_w, root_h = super().configure_best_screen_size()
         if self.touchpad_device:
             self.touchpad_device.root_w = root_w
             self.touchpad_device.root_h = root_h
@@ -125,18 +132,38 @@ class X11ServerBase(X11ServerCore):
     def init_dbus(self, dbus_pid, dbus_env):
         dbuslog("init_dbus(%s, %s)", dbus_pid, dbus_env)
         if dbus_pid and dbus_env:
-            os.environ.update(dbus_env)
+            os.environb.update(dbus_env)
             self.dbus_pid = dbus_pid
             self.dbus_env = dbus_env
             #now we can save values on the display
             #(we cannot access gtk3 until dbus has started up)
-            from xpra.server.dbus.dbus_start import save_dbus_env
-            save_dbus_env(dbus_env)
+            def _save_int(prop_name, pid):
+                root_prop_set(prop_name, "u32", pid)
+            def _save_str(prop_name, s):
+                root_prop_set(prop_name, "latin1", s)
+            #DBUS_SESSION_BUS_ADDRESS=unix:abstract=/tmp/dbus-B8CDeWmam9,guid=b77f682bd8b57a5cc02f870556cbe9e9
+            #DBUS_SESSION_BUS_PID=11406
+            #DBUS_SESSION_BUS_WINDOWID=50331649
+            for n,conv,save in (
+                    ("ADDRESS",     bytestostr,     _save_str),
+                    ("PID",         int,            _save_int),
+                    ("WINDOW_ID",   int,            _save_int)):
+                k = "DBUS_SESSION_BUS_%s" % n
+                v = dbus_env.get(k)
+                if v is None:
+                    continue
+                try:
+                    tv = conv(v)
+                    save(k, tv)
+                except Exception as e:
+                    log("save_dbus_env(%s)", dbus_env, exc_info=True)
+                    log.error("failed to save dbus environment variable '%s' with value '%s':\n" % (k, v))
+                    log.error(" %s\n" % e)
 
 
     def last_client_exited(self):
         self.reset_settings()
-        X11ServerCore.last_client_exited(self)
+        super().last_client_exited()
 
     def init_virtual_devices(self, devices):
         #(this runs in the main thread - before the main loop starts)
@@ -228,9 +255,8 @@ class X11ServerBase(X11ServerCore):
         screenlog("set_icc_profile() icc data for %s: %s (%i bytes)",
                   ui_clients[0], hexstr(data or ""), len(data or ""))
         self.icc_profile = data
-        from xpra.x11.gtk_x11.prop import prop_set
-        prop_set(self.root_window, "_ICC_PROFILE", ["u32"], [ord(x) for x in data])
-        prop_set(self.root_window, "_ICC_PROFILE_IN_X_VERSION", "u32", 0*100+4) #0.4 -> 0*100+4*1
+        root_prop_set("_ICC_PROFILE", ["u32"], [ord(x) for x in data])
+        root_prop_set("_ICC_PROFILE_IN_X_VERSION", "u32", 0*100+4) #0.4 -> 0*100+4*1
 
     def reset_icc_profile(self):
         screenlog("reset_icc_profile()")
@@ -261,14 +287,14 @@ class X11ServerBase(X11ServerCore):
         #almost like update_all, except we use the default_dpi,
         #since this is called before the first client connects
         self.do_update_server_settings({
-            b"resource-manager"  : b"",
-            b"xsettings-blob"    : (0, [])
+            "resource-manager"  : b"",
+            "xsettings-blob"    : (0, [])
             }, reset = True, dpi = self.default_dpi, cursor_size=24)
 
     def update_all_server_settings(self, reset=False):
         self.update_server_settings({
-            b"resource-manager"  : b"",
-            b"xsettings-blob"    : (0, []),
+            "resource-manager"  : b"",
+            "xsettings-blob"    : (0, []),
             }, reset=reset)
 
     def update_server_settings(self, settings=None, reset=False):
@@ -276,7 +302,7 @@ class X11ServerBase(X11ServerCore):
                                 self.dpi, self.double_click_time, self.double_click_distance, self.antialias, self.cursor_size)
 
     def do_update_server_settings(self, settings, reset=False,
-                                  dpi=0, double_click_time=0, double_click_distance=(-1, -1), antialias={}, cursor_size=-1):
+                                  dpi=0, double_click_time=0, double_click_distance=(-1, -1), antialias=None, cursor_size=-1):
         if not self._xsettings_enabled:
             log("ignoring xsettings update: %s", settings)
             return
@@ -298,10 +324,12 @@ class X11ServerBase(X11ServerCore):
         log("overrides: dpi=%s, double click time=%s, double click distance=%s",
             dpi, double_click_time, double_click_distance)
         log("overrides: antialias=%s", antialias)
+        #older versions may send keys as "bytes":
+        settings = dict((bytestostr(k), v) for k,v in settings.items())
         self._settings.update(settings)
         for k, v in settings.items():
             #cook the "resource-manager" value to add the DPI and/or antialias values:
-            if k==b"resource-manager" and (dpi>0 or antialias or cursor_size>0):
+            if k=="resource-manager" and (dpi>0 or antialias or cursor_size>0):
                 value = bytestostr(v)
                 #parse the resources into a dict:
                 values={}
@@ -346,12 +374,12 @@ class X11ServerBase(X11ServerCore):
                 for vk, vv in values.items():
                     value += "%s:\t%s\n" % (vk, vv)
                 #record the actual value used
-                self._settings[b"resource-manager"] = value
+                self._settings["resource-manager"] = value
                 v = value.encode("utf-8")
 
             #cook xsettings to add various settings:
             #(as those may not be present in xsettings on some platforms.. like win32 and osx)
-            if k==b"xsettings-blob" and \
+            if k=="xsettings-blob" and \
             (self.double_click_time>0 or self.double_click_distance!=(-1, -1) or antialias or dpi>0):
                 #start by removing blacklisted options:
                 def filter_blacklisted():
@@ -367,42 +395,42 @@ class X11ServerBase(X11ServerCore):
                 def set_xsettings_value(name, value_type, value):
                     #remove existing one, if any:
                     serial, values = v
-                    new_values = [(_t,_n,_v,_s) for (_t,_n,_v,_s) in values if _n!=name]
-                    new_values.append((value_type, name, value, 0))
+                    bn = name.encode("utf-8")
+                    new_values = [(_t,_n,_v,_s) for (_t,_n,_v,_s) in values if _n!=bn]
+                    new_values.append((value_type, bn, value, 0))
                     return serial, new_values
                 def set_xsettings_int(name, value):
                     if value<0: #not set, return v unchanged
                         return v
                     return set_xsettings_value(name, XSettingsTypeInteger, value)
                 if dpi>0:
-                    v = set_xsettings_int(b"Xft/DPI", dpi*1024)
+                    v = set_xsettings_int("Xft/DPI", dpi*1024)
                 if double_click_time>0:
-                    v = set_xsettings_int(b"Net/DoubleClickTime", self.double_click_time)
+                    v = set_xsettings_int("Net/DoubleClickTime", self.double_click_time)
                 if antialias:
                     ad = typedict(antialias)
-                    v = set_xsettings_int(b"Xft/Antialias",  ad.intget("enabled", -1))
-                    v = set_xsettings_int(b"Xft/Hinting",    ad.intget("hinting", -1))
-                    v = set_xsettings_value(b"Xft/RGBA",     XSettingsTypeString, ad.strget("orientation", "none").lower())
-                    v = set_xsettings_value(b"Xft/HintStyle", XSettingsTypeString, _get_antialias_hintstyle(ad))
+                    v = set_xsettings_int("Xft/Antialias",  ad.intget("enabled", -1))
+                    v = set_xsettings_int("Xft/Hinting",    ad.intget("hinting", -1))
+                    v = set_xsettings_value("Xft/RGBA",     XSettingsTypeString, ad.strget("orientation", "none").lower())
+                    v = set_xsettings_value("Xft/HintStyle", XSettingsTypeString, _get_antialias_hintstyle(ad))
                 if double_click_distance!=(-1, -1):
                     #some platforms give us a value for each axis,
                     #but X11 only has one, so take the average
                     try:
                         x,y = double_click_distance
                         if x>0 and y>0:
-                            d = iround((x+y)/2.0)
+                            d = round((x+y)/2)
                             d = max(1, min(128, d))     #sanitize it a bit
-                            v = set_xsettings_int(b"Net/DoubleClickDistance", d)
+                            v = set_xsettings_int("Net/DoubleClickDistance", d)
                     except Exception as e:
                         log.warn("error setting double click distance from %s: %s", double_click_distance, e)
 
             if k not in old_settings or v != old_settings[k]:
-                if k == b"xsettings-blob":
+                if k == "xsettings-blob":
                     self.set_xsettings(v)
-                elif k == b"resource-manager":
-                    from xpra.x11.gtk_x11.prop import prop_set
+                elif k == "resource-manager":
                     p = "RESOURCE_MANAGER"
                     log("server_settings: setting %s to %r", p, v)
-                    prop_set(self.root_window, p, "latin1", strtobytes(v).decode("latin1"))
+                    root_prop_set(p, "latin1", strtobytes(v).decode("latin1"))
                 else:
                     log.warn("Warning: unexpected setting '%s'", bytestostr(k))
