@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import threading
+from time import monotonic
 from gi.repository import Gdk
 
 from xpra.x11.bindings.core_bindings import set_context_check, X11CoreBindings     #@UnresolvedImport
@@ -16,15 +17,15 @@ from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@Unresolved
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
 from xpra.gtk_common.error import XError, xswallow, xsync, xlog, trap, verify_sync
 from xpra.gtk_common.gtk_util import get_default_root_window
-from xpra.server.server_uuid import save_uuid, get_uuid
+from xpra.server.server_uuid import save_uuid, get_uuid, save_mode
 from xpra.x11.vfb_util import parse_resolution
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
-from xpra.x11.gtk_x11.prop import prop_get, prop_set
+from xpra.x11.gtk_x11.prop import prop_get, prop_set, prop_del
 from xpra.x11.gtk_x11.gdk_display_source import close_gdk_display_source
 from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter, cleanup_x11_filter, cleanup_all_event_receivers
 from xpra.common import MAX_WINDOW_SIZE
-from xpra.os_util import monotonic_time, strtobytes
-from xpra.util import typedict, iround, envbool, first_time, XPRA_DPI_NOTIFICATION_ID
+from xpra.os_util import strtobytes
+from xpra.util import typedict, envbool, first_time, XPRA_DPI_NOTIFICATION_ID
 from xpra.net.compression import Compressed
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.x11.xkbhelper import clean_keyboard_state
@@ -52,7 +53,7 @@ FAKE_X11_INIT_ERROR = envbool("XPRA_FAKE_X11_INIT_ERROR", False)
 
 
 class XTestPointerDevice:
-
+    __slots__ = ()
     def __repr__(self):
         return "XTestPointerDevice"
 
@@ -66,9 +67,6 @@ class XTestPointerDevice:
         with xsync:
             X11Keyboard.xtest_fake_button(button, pressed)
 
-    def close(self):
-        pass
-
     def has_precise_wheel(self):
         return False
 
@@ -87,12 +85,11 @@ class X11ServerCore(GTKServerBase):
         self.touchpad_device = None
         self.pointer_device_map = {}
         self.keys_pressed = {}
-        self.last_mouse_user = None
         self.libfakeXinerama_so = None
         self.initial_resolution = None
         self.x11_filter = False
         self.randr_sizes_added = []
-        GTKServerBase.__init__(self)
+        super().__init__()
         log("XShape=%s", X11Window.displayHasXShape())
 
     def init(self, opts):
@@ -147,7 +144,7 @@ class X11ServerCore(GTKServerBase):
             self.save_mode()
 
     def save_mode(self):
-        prop_set(get_default_root_window(), "XPRA_SERVER_MODE", "latin1", self.get_server_mode())
+        save_mode(self.get_server_mode())
 
     def init_fake_xinerama(self):
         if self.fake_xinerama in FALSE_OPTIONS:
@@ -253,7 +250,7 @@ class X11ServerCore(GTKServerBase):
             keylog.error(" %s", e)
 
     def init_packet_handlers(self):
-        GTKServerBase.init_packet_handlers(self)
+        super().init_packet_handlers()
         self.add_packet_handler("force-ungrab", self._process_force_ungrab)
         self.add_packet_handler("wheel-motion", self._process_wheel_motion)
 
@@ -270,6 +267,7 @@ class X11ServerCore(GTKServerBase):
         return env
 
     def do_cleanup(self):
+        log("do_cleanup() x11_filter=%s", self.x11_filter)
         if self.x11_filter:
             self.x11_filter = False
             cleanup_x11_filter()
@@ -282,18 +280,33 @@ class X11ServerCore(GTKServerBase):
                     with xsync:
                         cleanup_all_event_receivers()
                         #all went well, we're done
+                        log("all event receivers have been removed")
                         break
                 except Exception as e:
                     l("failed to remove event receivers: %s", e)
         if self.fake_xinerama:
             cleanup_fakeXinerama()
-        with xswallow:
+        with xlog:
             clean_keyboard_state()
-        GTKServerBase.do_cleanup(self)
+        #prop_del does its own xsync:
+        self.clean_x11_properties()
+        super().do_cleanup()
         log("close_gdk_display_source()")
         close_gdk_display_source()
 
 
+    def clean_x11_properties(self):
+        self.do_clean_x11_properties("XPRA_SERVER_MODE", "_XPRA_RANDR_EXACT_SIZE")
+
+    def do_clean_x11_properties(self, *properties):
+        root = get_default_root_window()
+        for prop in properties:
+            try:
+                prop_del(root, prop)
+            except Exception as e:
+                log("prop_del(%s, %s) %s", root, prop, e)
+
+
     def get_uuid(self):
         return get_uuid()
 
@@ -325,7 +338,8 @@ class X11ServerCore(GTKServerBase):
                     "force_ungrab"              : True,
                     "keyboard.fast-switching"   : True,
                     "wheel.precise"             : self.pointer_device.has_precise_wheel(),
-                    "touchpad-device"              : bool(self.touchpad_device),
+                    "pointer.optional"          : True,
+                    "touchpad-device"           : bool(self.touchpad_device),
                     })
             if self.randr:
                 sizes = self.get_all_screen_sizes()
@@ -336,7 +350,7 @@ class X11ServerCore(GTKServerBase):
         return capabilities
 
     def do_get_info(self, proto, server_sources) -> dict:
-        start = monotonic_time()
+        start = monotonic()
         info = super().do_get_info(proto, server_sources)
         sinfo = info.setdefault("server", {})
         sinfo.update({
@@ -344,7 +358,7 @@ class X11ServerCore(GTKServerBase):
             "fakeXinerama"          : bool(self.libfakeXinerama_so),
             "libfakeXinerama"       : self.libfakeXinerama_so or "",
             })
-        log("X11ServerCore.do_get_info took %ims", (monotonic_time()-start)*1000)
+        log("X11ServerCore.do_get_info took %ims", (monotonic()-start)*1000)
         return info
 
     def get_ui_info(self, proto, wids=None, *args) -> dict:
@@ -378,7 +392,7 @@ class X11ServerCore(GTKServerBase):
         if self.randr:
             with xlog:
                 sizes = self.get_all_screen_sizes()
-                if len(sizes)>=0:
+                if sizes:
                     sinfo["randr"] = {
                         ""          : True,
                         "options"   : tuple(reversed(sorted(sizes))),
@@ -521,16 +535,13 @@ class X11ServerCore(GTKServerBase):
         if not self.randr:
             return root_w, root_h
         sss = tuple(x for x in self._server_sources.values() if x.ui_client)
-        if len(sss)>1:
-            screenlog.info("screen used by %i clients:", len(sss))
         bigger = True
         max_w, max_h = 0, 0
         min_w, min_h = 16384, 16384
+        client_sizes = {}
         for ss in sss:
             client_size = ss.desktop_size
-            if not client_size:
-                size = "unknown"
-            else:
+            if client_size:
                 w, h = client_size
                 size = "%ix%i" % (w, h)
                 max_w = max(max_w, w)
@@ -540,8 +551,12 @@ class X11ServerCore(GTKServerBase):
                 if h>0:
                     min_h = min(min_h, h)
                 bigger = bigger and ss.screen_resize_bigger
-            if len(sss)>1:
-                screenlog.info("* %s: %s", ss.uuid, size)
+                client_sizes[ss.uuid] = size
+        if len(client_sizes)>1:
+            screenlog.info("screen used by %i clients:", len(client_sizes))
+            for uuid, size in client_sizes.items():
+                screenlog.info("* %s: %s", uuid, size)
+            bigger = True
         if bigger:
             w, h = max_w, max_h
         else:
@@ -556,7 +571,9 @@ class X11ServerCore(GTKServerBase):
         return self.set_screen_size(w, h, bigger)
 
     def get_best_screen_size(self, desired_w, desired_h, bigger=True):
-        return self.do_get_best_screen_size(desired_w, desired_h, bigger)
+        r = self.do_get_best_screen_size(desired_w, desired_h, bigger)
+        screenlog("get_best_screen_size%s=%s", (desired_w, desired_h, bigger), r)
+        return r
 
     def do_get_best_screen_size(self, desired_w, desired_h, bigger=True):
         if not self.randr:
@@ -633,7 +650,8 @@ class X11ServerCore(GTKServerBase):
             client_w, client_h = 0, 0
             sss = self._server_sources.values()
             for ss in sss:
-                for s in ss.screen_sizes:
+                screen_sizes = getattr(ss, "screen_sizes", ())
+                for s in screen_sizes:
                     if len(s)>=10:
                         #(display_name, width, height, width_mm, height_mm, monitors,
                         # work_x, work_y, work_width, work_height)
@@ -643,8 +661,8 @@ class X11ServerCore(GTKServerBase):
                         hmm = max(hmm, s[4])
             if wmm>0 and hmm>0 and client_w>0 and client_h>0:
                 #calculate "real" dpi:
-                xdpi = iround(client_w * 25.4 / wmm)
-                ydpi = iround(client_h * 25.4 / hmm)
+                xdpi = round(client_w * 25.4 / wmm)
+                ydpi = round(client_h * 25.4 / hmm)
                 screenlog("calculated DPI: %s x %s (from w: %s / %s, h: %s / %s)",
                           xdpi, ydpi, client_w, wmm, client_h, hmm)
         self.set_dpi(xdpi, ydpi)
@@ -718,8 +736,8 @@ class X11ServerCore(GTKServerBase):
             def show_dpi():
                 wmm, hmm = RandR.get_screen_size_mm()      #ie: (1280, 1024)
                 screenlog("RandR.get_screen_size_mm=%s,%s", wmm, hmm)
-                actual_xdpi = iround(root_w * 25.4 / wmm)
-                actual_ydpi = iround(root_h * 25.4 / hmm)
+                actual_xdpi = round(root_w * 25.4 / wmm)
+                actual_ydpi = round(root_h * 25.4 / hmm)
                 if abs(actual_xdpi-xdpi)<=1 and abs(actual_ydpi-ydpi)<=1:
                     screenlog.info("DPI set to %s x %s", actual_xdpi, actual_ydpi)
                     screenlog("wanted: %s x %s", xdpi, ydpi)
@@ -751,6 +769,9 @@ class X11ServerCore(GTKServerBase):
             if first_time("DPI-warning-%s" % ss.uuid):
                 sources[0].may_notify(XPRA_DPI_NOTIFICATION_ID, "DPI Issue", body, icon_name="font")
 
+    def set_dpi(self, xdpi, ydpi):
+        """ overriden in the seamless server """
+
 
     def _process_server_settings(self, _proto, packet):
         settings = packet[1]
@@ -845,10 +866,6 @@ class X11ServerCore(GTKServerBase):
         return -1
 
 
-    def cleanup_input_devices(self):
-        pass
-
-
     def setup_input_devices(self):
         from xpra.server import server_features
         xinputlog("setup_input_devices() input_devices feature=%s", server_features.input_devices)
@@ -893,9 +910,13 @@ class X11ServerCore(GTKServerBase):
 
     def _process_wheel_motion(self, proto, packet):
         assert self.pointer_device.has_precise_wheel()
+        ss = self.get_server_source(proto)
+        if not ss:
+            return
         wid, button, distance, pointer, modifiers, _buttons = packet[1:7]
         with xsync:
             if self.do_process_mouse_common(proto, wid, pointer):
+                self.last_mouse_user = ss.uuid
                 self._update_modifiers(proto, wid, modifiers)
                 self.pointer_device.wheel_motion(button, distance/1000.0)   #pylint: disable=no-member
 
@@ -940,18 +961,12 @@ class X11ServerCore(GTKServerBase):
     def do_process_mouse_common(self, proto, wid, pointer, deviceid=-1, *args):
         mouselog("do_process_mouse_common%s", tuple([proto, wid, pointer, deviceid]+list(args)))
         if self.readonly:
-            return None
+            return False
         pos = self.root_window.get_pointer()[-3:-1]
-        uuid = None
-        if proto:
-            ss = self.get_server_source(proto)
-            if ss:
-                uuid = ss.uuid
-        if pos!=pointer[:2] or self.input_devices=="xi":
-            self.last_mouse_user = uuid
+        if (pointer and pos!=pointer[:2]) or self.input_devices=="xi":
             with xswallow:
                 self._move_pointer(wid, pointer, deviceid, *args)
-        return pointer
+        return True
 
     def _update_modifiers(self, proto, wid, modifiers):
         if self.readonly:
